{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Cookies;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, System.SysUtils;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils;
  {$ENDIF}


type
  TCookie = class(TCollectionItem)
  private
    FPath: string;
    FName: string;
    FExpiry: TDateTime;
    FValue: string;
    FChanged: boolean;
  protected
    property Changed: boolean read FChanged write FChanged;
  public
    function CookieAsString: string;
  published
    property Name: string read FName write FName;
    property Value: string read FValue write FValue;
    property Expiry: TDateTime read FExpiry write FExpiry;
    property Path: string read FPath write FPath;
  end;

  TWebCookie = class(TCookie);

  TCookies = class(TCollection)
  private
    function GetItem(Index: integer): TCookie;
    procedure SetItem(Index: integer; const Value: TCookie);
  public
    constructor Create;

    procedure GetCookies;
    procedure SetCookies;

    procedure Delete(ACookie: TCookie); overload;
    procedure Delete(const AName: string); overload;

    function Add(const AName, AValue: string; Expiry: TDateTime): TCookie; overload;
    function Add(const AName, AValue: string): TCookie; overload;
    function Add(const AName, AValue, APath: string): TCookie; overload;
    function Add(const AName, AValue, APath: string; Expiry: TDateTime): TCookie; overload;
    property Items[Index: integer]: TCookie read GetItem write SetItem;
    function Find(const AName: string): TCookie;
  end;

  TWebCookies = class(TCookies);

implementation


{ TCookies }

function TCookies.Add(const AName, AValue: string; Expiry: TDateTime): TCookie;
begin
  Result := Add(AName, AValue, '/', Expiry);
end;

function TCookies.Add(const AName, AValue: string): TCookie;
begin
  Result := Add(AName, AValue, '/', 0);
end;

function TCookies.Add(const AName, AValue, APath: string): TCookie;
begin
  Result := Add(AName, AValue, APath, 0);
end;

function TCookies.Add(const AName, AValue, APath: string;
  Expiry: TDateTime): TCookie;
begin
  Result := nil;

  if AName = '' then
    Exit;

  Result := TCookie(inherited Add);
  Result.Name := AName;
  Result.Value := AValue;
  Result.Expiry := Expiry;
  Result.Path := APath;
  Result.Changed := true;
end;

constructor TCookies.Create;
begin
  inherited Create(TCookie);
end;

procedure TCookies.Delete(const AName: string);
begin
end;

procedure TCookies.Delete(ACookie: TCookie);
begin
end;

function TCookies.Find(const AName: string): TCookie;
begin
  Result := nil;
end;

procedure TCookies.GetCookies;
begin
end;

function TCookies.GetItem(Index: integer): TCookie;
begin
  Result := TCookie(inherited Items[Index]);
end;

procedure TCookies.SetCookies;
begin
end;

procedure TCookies.SetItem(Index: integer; const Value: TCookie);
begin
  inherited Items[index] := Value;
end;

{ TCookie }

function TCookie.CookieAsString: string;
begin
  Result := Name + '=' + Value + ';';
end;

end.
