{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Electron;

interface

uses
  Classes, SysUtils, Vcl.Forms, WEBLib.Controls, WEBLib.Menus, js, jsdelphisystem, web, libelectron;

type
  TFormClass = class of TForm;

  TReceivedValue = record
    AsString: string;
    AsBoolean: Boolean;
    AsInteger: NativeInt;
    AsFloat: Double;
    AsObject: TJSObject;
    AsArray: TJSArray;
//    ValueType: TJSValueType;
  end;

  TElectronStringList = class(TStringList)
  public
    procedure LoadFromFile(const FileName: string); override;
    procedure SaveToFile(const FileName: string); override;
  end;

  TElectronBinaryDataStream = class(TPersistent)
  private
    FText: string;
    FBase64: string;
    FAsArrayBuffer: TJSArrayBuffer;
  public
    procedure LoadFromFile(const FileName: string); virtual;
    procedure SaveToFile(const FileName: string); virtual;
    property AsArrayBuffer: TJSArrayBuffer read FAsArrayBuffer write FAsArrayBuffer;
    property Base64: string read FBase64 write FBase64;
    property Text: string read FText write FText;
  end;

  TElectronDialogOption = (openFile, openDirectory, multiSelections, showHiddenFiles);
  TElectronDialogOptions = set of TElectronDialogOption;
  TElectronMessageBoxType = (embNone, embInfo, embError, embQuestion, embWarning);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronOpenDialog = class(TComponent)
  private
    FButtonLabel: string;
    FDefaultPath: string;
    FFileName: string;
    FFilters: string;
    FOptions: TElectronDialogOptions;
    FTitle: string;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute(AProc: TSelectOpenFileCallBack): Boolean;
  published
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property FileName: string read FFileName write FFileName;
    property Filters: string read FFilters write FFilters;
    property Options: TElectronDialogOptions read FOptions write FOptions default [openFile];
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronSaveDialog = class(TComponent)
  private
    FButtonLabel: string;
    FDefaultPath: string;
    FFileName: string;
    FFilters: string;
    FTitle: string;
  public
    constructor Create(AOwner: TComponent); override;
    function Execute(AProc: TSelectSaveFileCallBack): Boolean;
  published
    property ButtonLabel: string read FButtonLabel write FButtonLabel;
    property DefaultPath: string read FDefaultPath write FDefaultPath;
    property FileName: string read FFileName write FFileName;
    property Filters: string read FFilters write FFilters;
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronMessageBox = class(TComponent)
  private
    FButtons: TStringList;
    FCheckboxChecked: Boolean;
    FCheckboxLabel: string;
    FCancelId: NativeInt;
    FDefaultId: NativeInt;
    FDetail: string;
    FIconURL: string;
    FIconPath: string;
    FMessage: string;
    FNoLink: Boolean;
    FNormalizeAccessKeys: Boolean;
    FResponse: NativeInt;
    FTitle: string;
    FType: TElectronMessageBoxType;
    procedure SetButtons(const Value: TStringList);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute(AProc: TSelectMessageBoxCallBack): Boolean;
  published
    property Buttons: TStringList read FButtons write SetButtons;
    property CancelId: NativeInt read FCancelId write FCancelId default 0;
    property CheckboxChecked: Boolean read FCheckboxChecked write FCheckboxChecked default False;
    property CheckboxLabel: string read FCheckboxLabel write FCheckboxLabel;
    property DefaultId: NativeInt read FDefaultId write FDefaultId default 0;
    property Detail: string read FDetail write FDetail;
    property DialogType: TElectronMessageBoxType read FType write FType default embNone;
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Message: string read FMessage write FMessage;
    property NoLink: Boolean read FNoLink write FNoLink default False;
    property NormalizeAccessKeys: Boolean read FNormalizeAccessKeys write FNormalizeAccessKeys default False;
    property Response: NativeInt read FResponse write FResponse default 0;
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronErrorBox = class(TComponent)
  private
    FContent: string;
    FTitle: string;
  public
    procedure Execute;
  published
    property Content: string read FContent write FContent;
    property Title: string read FTitle write FTitle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronMainMenu = class(TWebCustomMainMenu)
  public
    procedure EndUpdate;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronPopupMenu = class(TWebPopupMenu)
  public
    procedure EndUpdate;
  end;

  TElectronClipboard = class(TPersistent)
  private
    FAsRTF: string;
    FAsImageURL: string;
    FAsHTML: string;
    FAsText: string;
    FFormatCount: Integer;
    function GetFormats(Index: Integer): string;
  public
    procedure Clear;
    function HasFormat(AFormat: string): Boolean;
    property AsHTML: string read FAsHTML write FAsHTML;
    property AsImageURL: string read FAsImageURL write FAsImageURL;
    property AsRTF: string read FAsRTF write FAsRTF;
    property AsText: string read FAsText write FAsText;
    property FormatCount: Integer read FFormatCount;
    property Formats[Index: Integer]: string read GetFormats;
  end;

  TElectronShell = class(TPersistent)
  public
    procedure Beep;
    function OpenExternal(URL: string): Boolean;
    function OpenItem(FullPath: string): Boolean;
    function MoveItemToTrash(FullPath: string): Boolean;
    function ShowItemInFolder(FullPath: string): Boolean;
  end;

  TElectronBrowserWindowResizeEvent = procedure(Sender: TObject; Width: NativeInt; Height: NativeInt) of object;
  TElectronBrowserWindowMessageEvent = procedure(AMessage: TReceivedValue) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronBrowserWindow = class(TComponent)
  private
    FFullScreen: Boolean;
    FFullScreenable: Boolean;
    FIconPath: string;
    FIconURL: string;
    FKiosk: Boolean;
    FMaxHeight: Integer;
    FMaxWidth: Integer;
    FMinHeight: Integer;
    FMinWidth: Integer;
    FResizable: Boolean;
    FOnActivate: TNotifyEvent;
    FOnClose: TNotifyEvent;
    FOnDeactivate: TNotifyEvent;
    FOnExitFullScreen: TNotifyEvent;
    FOnFullScreen: TNotifyEvent;
    FOnHide: TNotifyEvent;
    FOnMaximize: TNotifyEvent;
    FOnMinimize: TNotifyEvent;
    FOnResize: TElectronBrowserWindowResizeEvent;
    FOnShow: TNotifyEvent;
    FFormClass: TFormClass;
    FOnMessage: TElectronBrowserWindowMessageEvent;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    procedure Close;
    procedure ForceClose;
    procedure Hide;
    procedure LoadFromURL(URL: string);
    procedure SendMessage(AMessage: string); overload;
    procedure SendMessage(AMessage: JSValue); overload;
    procedure SendMessage(Channel: string; AMessage: string); overload;
    procedure SendMessage(Channel: string; AMessage: JSValue); overload;
    procedure Show;
    procedure ShowModal;
    property FormClass: TFormClass read FFormClass write FFormClass;
  published
    property FullScreen: Boolean read FFullScreen write FFullScreen default False;
    property FullScreenable: Boolean read FFullScreenable write FFullScreenable default True;
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Kiosk: Boolean read FKiosk write FKiosk default False;
    property MaxHeight: Integer read FMaxHeight write FMaxHeight;
    property MaxWidth: Integer read FMaxWidth write FMaxWidth;
    property MinHeight: Integer read FMinHeight write FMinHeight default 0;
    property MinWidth: Integer read FMinWidth write FMinWidth default 0;
    property Resizable: Boolean read FResizable write FResizable default True;
    property OnActivate: TNotifyEvent read FOnActivate write FOnActivate;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnDeactivate: TNotifyEvent read FOnDeactivate write FOnDeactivate;
    property OnExitFullScreen: TNotifyEvent read FOnExitFullScreen write FOnExitFullScreen;
    property OnFullScreen: TNotifyEvent read FOnFullScreen write FOnFullScreen;
    property OnHide: TNotifyEvent read FOnHide write FOnHide;
    property OnMaximize: TNotifyEvent read FOnMaximize write FOnMaximize;
    property OnMessage: TElectronBrowserWindowMessageEvent read FOnMessage write FOnMessage;
    property OnMinimize: TNotifyEvent read FOnMinimize write FOnMinimize;
    property OnResize: TElectronBrowserWindowResizeEvent read FOnResize write FOnResize;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  TElectronWindow = class(TPersistent)
  public
    procedure CloseApp;
    procedure CloseWindow;
    procedure CloseDevTools;
    procedure Copy;
    procedure CopyImageAt(X, Y: NativeInt);
    procedure Cut;
    procedure Delete;
    procedure DownloadURL(URL: string);
    procedure OpenDevTools;
    procedure Paste;
    procedure Redo;
    procedure Reload;
    procedure Replace(AText: string);
    procedure ReplaceMisspelling(AText: string);
    procedure SelectAll;
    procedure ToggleDevTools;
    procedure Undo;
    procedure Unselect;
  end;

  TElectronIPCRenderer = class(TPersistent)
  public
    procedure Drag(PathToItem: string);
    procedure Listen(Channel: string; Listener: TIPCEventHandler);
    procedure ListenOnce(Channel: string; Listener: TIPCEventHandler);
    procedure RemoveAllListeners(Channel: string);
    procedure Send(Channel: string; JSObject: JSValue); overload;
    procedure Send(Channel: string; AMessage: string); overload;
  end;

  TElectronDragAndDrop = class(TPersistent)
  private
    FOnDrag: TNotifyEvent;
  public
    procedure StartDrag(PathToItem: string);
    procedure ListenToDrag(IconPath: string); overload;
    procedure RemoveListeners;
    property OnStartDrag: TNotifyEvent read FOnDrag write FOnDrag;
  end;

  TElectronPath = class(TPersistent)
  public
    function GetAppData: string;
    function GetAppPath: string;
    function GetDesktop: string;
    function GetDocuments: string;
    function GetDownloads: string;
    function GetExe: string;
    function GetHome: string;
    function GetMusic: string;
    function GetUserData: string;
    function GetPictures: string;
    function GetTemp: string;
    function GetVideos: string;
  end;

  TElectronIPCCommunicationReceiveEvent = procedure(AMessage: TReceivedValue) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronIPCCommunication = class(TComponent)
  private
    FChannel: string;
    FOnReceive: TElectronIPCCommunicationReceiveEvent;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Send(Channel: string; AMessage: JSValue); overload;
    procedure Send(Channel: string; AMessage: string); overload;
    procedure SendToParent(AMessage: JSValue);
  published
    property Channel: string read FChannel write FChannel;
    property OnReceive: TElectronIPCCommunicationReceiveEvent read FOnReceive write FOnReceive;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronTrayIcon = class(TComponent)
  private
    FIconPath: string;
    FIconURL: string;
    FMenu: TElectronPopupMenu;
    FToolTip: string;
    FOnClick: TNotifyEvent;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property IconPath: string read FIconPath write FIconPath;
    property IconURL: string read FIconURL write FIconURL;
    property Menu: TElectronPopupMenu read FMenu write FMenu;
    property ToolTip: string read FToolTip write FToolTip;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TElectronFileWatch = class(TCollectionItem)
  private
    FFileName: string;
    FOnChange: TNotifyEvent;
  public
    constructor Create(ACollection: TCollection); override;
    destructor Destroy; override;
  published
    property FileName: string read FFileName write FFileName;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  TElectronFileWatchClass = class of TElectronFileWatch; //?

  TElectronFileWatches = class(TOwnedCollection)
  private
    procedure SetItem(Index: Integer; const Value: TElectronFileWatch);
    function GetItem(Index: Integer): TElectronFileWatch;
  public
    function Add: TElectronFileWatch;
    property Items[Index: Integer]: TElectronFileWatch read GetItem write SetItem;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronFileWatcher = class(TComponent)
  private
    FFiles: TElectronFileWatches;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Files: TElectronFileWatches read FFiles write FFiles;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronGlobalShortcut = class(TComponent)
  private
    FOnShortcut: TNotifyEvent;
    FShortcut: TShortCut;
  published
    property ShortCut: TShortCut read FShortcut write FShortcut;
    property OnShortcut: TNotifyEvent read FOnShortcut write FOnShortcut;
  end;

  TElectronNotificationDirection = (endAuto, endLTR, endRTL);

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TElectronNotification = class(TComponent)
  private
    FDirection: TElectronNotificationDirection;
    FImage: string;
    FOnNotificationClick: TNotifyEvent;
    FBody: string;
    FIcon: string;
    FTitle: string;
  public
    procedure Show;
  published
    property Body: string read FBody write FBody;
    property Direction: TElectronNotificationDirection read FDirection write FDirection;
    property IconURL: string read FIcon write FIcon;
    property ImageURL: string read FImage write FImage;
    property Title: string read FTitle write FTitle;
    property OnNotificationClick: TNotifyEvent read FOnNotificationClick write FOnNotificationClick;
  end;

function CreateBlobFromFile(FileName: string): TJSBlob;
function CreateUint8ArrayFromFile(FileName: string): TJSUint8Array;
procedure DownloadToFolder(URL, Path: string);
function ElectronDragAndDrop: TElectronDragAndDrop;
function ElectronClipboard: TElectronClipboard;
function ElectronShell: TElectronShell;
function ElectronIPCRenderer: TElectronIPCRenderer;
function ElectronPath: TElectronPath;
function ElectronWindow: TElectronWindow;
function GetElectronFileList(Source: TJSEvent) : TJSElectronFileList;
function ShortCutToText(ShortCut: TShortCut): string;

implementation

{ TElectronStringList }

procedure TElectronStringList.LoadFromFile(const FileName: string);
begin
  //
end;

procedure TElectronStringList.SaveToFile(const FileName: string);
begin
  //
end;

{ TElectronOpenDialog }

constructor TElectronOpenDialog.Create(AOwner: TComponent);
begin
  inherited;
  FOptions := [openFile];
end;

function TElectronOpenDialog.Execute(AProc: TSelectOpenFileCallBack): boolean;
begin
  Result := true;
end;

{ TElectronSaveDialog }

constructor TElectronSaveDialog.Create(AOwner: TComponent);
begin
  inherited;
end;

function TElectronSaveDialog.Execute(AProc: TSelectSaveFileCallBack): boolean;
begin
  Result := true;
end;

{ TElectronMessageBox }

constructor TElectronMessageBox.Create(AOwner: TComponent);
begin
  inherited;
  FButtons := TStringList.Create;
end;

destructor TElectronMessageBox.Destroy;
begin
  FButtons.Free;
  inherited;
end;

function TElectronMessageBox.Execute(AProc: TSelectMessageBoxCallBack): boolean;
begin
  Result := True;
end;

procedure TElectronMessageBox.SetButtons(const Value: TStringList);
begin
  FButtons.Assign(Value);
end;

{ TElectronErrorBox }

procedure TElectronErrorBox.Execute;
begin
  //
end;


{ TElectronBrowserWindow }

procedure TElectronBrowserWindow.Close;
begin
  //
end;

constructor TElectronBrowserWindow.Create(AOwner: TComponent);
begin
  inherited;
  FFullScreen := False;
  FFullScreenable := True;
  FResizable := True;
  FMinHeight := 0;
  FMinWidth := 0;
  FKiosk := False;
end;

procedure TElectronBrowserWindow.ForceClose;
begin
  //
end;

procedure TElectronBrowserWindow.Hide;
begin

end;

procedure TElectronBrowserWindow.LoadFromURL(URL: string);
begin
  //
end;

procedure TElectronBrowserWindow.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
end;

procedure TElectronBrowserWindow.SendMessage(AMessage: string);
begin
  //
end;

procedure TElectronBrowserWindow.SendMessage(AMessage: JSValue);
begin
  //
end;

procedure TElectronBrowserWindow.SendMessage(Channel: string; AMessage: JSValue);
begin
  //
end;

procedure TElectronBrowserWindow.Show;
begin
  //
end;

procedure TElectronBrowserWindow.ShowModal;
begin
  //
end;

procedure TElectronBrowserWindow.SendMessage(Channel, AMessage: string);
begin
  //
end;

{ TElectronTrayIcon }

constructor TElectronTrayIcon.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TElectronTrayIcon.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if not (csDestroying in ComponentState) then
  begin
    if (Operation = opRemove) then
    begin
      if (AComponent = FMenu) then
        FMenu := nil;
    end;
  end;
end;

{ TElectronIPCCommunication }

constructor TElectronIPCCommunication.Create(AOwner: TComponent);
begin
  inherited;
end;

procedure TElectronIPCCommunication.Send(Channel: string; AMessage: JSValue);
begin
  //
end;

procedure TElectronIPCCommunication.Send(Channel, AMessage: string);
begin
  //
end;

procedure TElectronIPCCommunication.SendToParent(AMessage: JSValue);
begin
  //
end;

{ TElectronFileWatch }

constructor TElectronFileWatch.Create(ACollection: TCollection);
begin
  inherited Create(ACollection);
end;

destructor TElectronFileWatch.Destroy;
begin
  inherited;
end;

{ TElectronFileWatches }

function TElectronFileWatches.Add: TElectronFileWatch;
begin
  Result := TElectronFileWatch(inherited Add);
end;

function TElectronFileWatches.GetItem(Index: Integer): TElectronFileWatch;
begin
  Result := TElectronFileWatch(inherited Items[Index]);
end;

procedure TElectronFileWatches.SetItem(Index: Integer;
  const Value: TElectronFileWatch);
begin
  inherited Items[Index] := Value;
end;

{ TElectronFileWatcher }

constructor TElectronFileWatcher.Create(AOwner: TComponent);
begin
  inherited;
  FFiles := TElectronFileWatches.Create(Self, TElectronFileWatch);
end;

destructor TElectronFileWatcher.Destroy;
begin
  FFiles.Free;
  inherited;
end;

{ TElectronBinaryDataStream }

procedure TElectronBinaryDataStream.LoadFromFile(const FileName: string);
begin
  //
end;

procedure TElectronBinaryDataStream.SaveToFile(const FileName: string);
begin
  //
end;

{ TElectronClipboard }

procedure TElectronClipboard.Clear;
begin
  //
end;

function TElectronClipboard.GetFormats(Index: Integer): string;
begin
  Result := '';
end;

function TElectronClipboard.HasFormat(AFormat: string): Boolean;
begin
  Result := True;
end;

{ TElectronShell }

procedure TElectronShell.Beep;
begin
  //
end;

function TElectronShell.MoveItemToTrash(FullPath: string): Boolean;
begin
  Result := True;
end;

function TElectronShell.OpenExternal(URL: string): Boolean;
begin
  Result := True;
end;

function TElectronShell.OpenItem(FullPath: string): Boolean;
begin
  Result := True;
end;

function TElectronShell.ShowItemInFolder(FullPath: string): Boolean;
begin
  Result := True;
end;

{ TElectronIPCRenderer }

procedure TElectronIPCRenderer.Drag(PathToItem: string);
begin
  //
end;

procedure TElectronIPCRenderer.Listen(Channel: string;
  Listener: TIPCEventHandler);
begin
  //
end;

procedure TElectronIPCRenderer.ListenOnce(Channel: string;
  Listener: TIPCEventHandler);
begin
  //
end;

procedure TElectronIPCRenderer.RemoveAllListeners(Channel: string);
begin
  //
end;

procedure TElectronIPCRenderer.Send(Channel: string; JSObject: JSValue);
begin
  //
end;

procedure TElectronIPCRenderer.Send(Channel, AMessage: string);
begin
  //
end;

{ TElectronDragAndDrop }

procedure TElectronDragAndDrop.ListenToDrag(IconPath: string);
begin
  //
end;

procedure TElectronDragAndDrop.RemoveListeners;
begin

end;

procedure TElectronDragAndDrop.StartDrag(PathToItem: string);
begin
  //
end;

{ TElectronPath }

function TElectronPath.GetAppData: string;
begin
  Result := '';
end;

function TElectronPath.GetAppPath: string;
begin
  Result := '';
end;

function TElectronPath.GetDesktop: string;
begin
  Result := '';
end;

function TElectronPath.GetDocuments: string;
begin
  Result := '';
end;

function TElectronPath.GetDownloads: string;
begin
  Result := '';
end;

function TElectronPath.GetExe: string;
begin
  Result := '';
end;

function TElectronPath.GetHome: string;
begin
  Result := '';
end;

function TElectronPath.GetMusic: string;
begin
  Result := '';
end;

function TElectronPath.GetPictures: string;
begin
  Result := '';
end;

function TElectronPath.GetTemp: string;
begin
  Result := '';
end;

function TElectronPath.GetUserData: string;
begin
  Result := '';
end;

function TElectronPath.GetVideos: string;
begin
  Result := '';
end;

{ TElectronWindow }

procedure TElectronWindow.CloseApp;
begin
  //
end;

procedure TElectronWindow.CloseDevTools;
begin
  //
end;

procedure TElectronWindow.CloseWindow;
begin
  //
end;

procedure TElectronWindow.Copy;
begin
  //
end;

procedure TElectronWindow.CopyImageAt(X, Y: NativeInt);
begin
  //
end;

procedure TElectronWindow.Cut;
begin
  //
end;

procedure TElectronWindow.Delete;
begin
  //
end;

procedure TElectronWindow.DownloadURL(URL: string);
begin
  //
end;

//function TElectronWindow.GetURL: string;
//begin
//  Result := '';
//end;

procedure TElectronWindow.OpenDevTools;
begin
  //
end;

procedure TElectronWindow.Paste;
begin
  //
end;

procedure TElectronWindow.Redo;
begin
  //
end;

procedure TElectronWindow.Reload;
begin
  //
end;

procedure TElectronWindow.Replace(AText: string);
begin
  //
end;

procedure TElectronWindow.ReplaceMisspelling(AText: string);
begin
  //
end;

procedure TElectronWindow.SelectAll;
begin
  //
end;

procedure TElectronWindow.ToggleDevTools;
begin
  //
end;

procedure TElectronWindow.Undo;
begin
  //
end;

procedure TElectronWindow.Unselect;
begin
  //
end;

{ TElectronNotification }

procedure TElectronNotification.Show;
begin
  //
end;

function CreateBlobFromFile(FileName: string): TJSBlob;
begin
  Result := nil;
end;

function CreateUint8ArrayFromFile(FileName: string): TJSUint8Array;
begin
  Result := nil;
end;

procedure DownloadToFolder(URL, Path: string);
begin
  //
end;

function ElectronDragAndDrop: TElectronDragAndDrop;
begin
  Result := nil;
end;

function ElectronClipboard: TElectronClipboard;
begin
  Result := nil;
end;

function ElectronShell: TElectronShell;
begin
  Result := nil;
end;

function ElectronIPCRenderer: TElectronIPCRenderer;
begin
  Result := nil;
end;

function ElectronPath: TElectronPath;
begin
  Result := nil;
end;

function ElectronWindow: TElectronWindow;
begin
  Result := nil;
end;

function GetElectronFileList(Source: TJSEvent) : TJSElectronFileList;
begin
  Result := nil;
end;

function ShortCutToText(ShortCut: TShortCut): string;
begin
  Result := '';
end;

{ TElectronMainMenu }

procedure TElectronMainMenu.EndUpdate;
begin

end;

{ TElectronPopupMenu }

procedure TElectronPopupMenu.EndUpdate;
begin

end;

end.
