{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.ElectronCDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, Web, jsdelphisystem, js, WEBLib.CDS;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, Web, jsdelphisystem, WEBLib.CDS;
  {$ENDIF}

type
  TElectronBaseErrorEvent = procedure(AErrorMessage: string) of object;

  TElectronNodeTLSSocketOptions = class(TPersistent)
  private
    FKeyPath: string;
    FMaxVersion: string;
    FCRLPath: string;
    FSSLProfile: string;
    FECDHCurve: string;
    FCertPath: string;
    FMinVersion: string;
    FCAPath: string;
    FPassphrase: string;
    FPFXPath: string;
    FClientCertEngine: string;
    FDHParam: string;
    FUseProfile: Boolean;
    FSigalgs: string;
    FRejectUnauthorized: Boolean;
  public
    constructor Create;
  published
    property UseProfile: Boolean read FUseProfile write FUseProfile default False;
    property SSLProfile: string read FSSLProfile write FSSLProfile;
    property RejectUnauthorized: Boolean read FRejectUnauthorized write FRejectUnauthorized default True;
    property CAPath: string read FCAPath write FCAPath;
    property CertPath: string read FCertPath write FCertPath;
    property Sigalgs: string read FSigalgs write FSigalgs;
    property ClientCertEngine: string read FClientCertEngine write FClientCertEngine;
    property CRLPath: string read FCRLPath write FCRLPath;
    property DHParam: string read FDHParam write FDHParam;
    property ECDHCurve: string read FECDHCurve write FECDHCurve;
    property KeyPath: string read FKeyPath write FKeyPath;
    property MaxVersion: string read FMaxVersion write FMaxVersion;
    property MinVersion: string read FMinVersion write FMinVersion;
    property Passphrase: string read FPassphrase write FPassphrase;
    property PFXPath: string read FPFXPath write FPFXPath;
  end;

  TElectronBaseConnection = class(TComponent)
  private
    FPort: Integer;
    FDatabaseName: string;
    FPassword: string;
    FHost: string;
    FUser: string;
    FOnConnectionError: TElectronBaseErrorEvent;
    function IsDatabaseNameStored: Boolean;
    function IsHostStored: Boolean;
    function IsPasswordStored: Boolean;
    function IsUserStored: Boolean;
  protected
    property User: string read FUser write FUser stored IsUserStored;
    property Host: string read FHost write FHost stored IsHostStored;
    property Password: string read FPassword write FPassword stored IsPasswordStored;
    property DatabaseName: string read FDatabaseName write FDatabaseName stored IsDatabaseNameStored;
    property Port: Integer read FPort write FPort default 8080;
    property OnConnectionError: TElectronBaseErrorEvent read FOnConnectionError write FOnConnectionError;
  end;

  TElectronBaseClientDataSet = class(TWebClientDataSet)
  private
    FTableName: string;
    FIndexName: string;
    FOnError: TElectronBaseErrorEvent;
    function IsIndexNameStored: Boolean;
    function IsTableNameStored: Boolean;
  protected
    property IndexName: string read FIndexName write FIndexName stored IsIndexNameStored;
    property TableName: string read FTableName write FTableName stored IsTableNameStored;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Refresh(Force: Boolean=False); reintroduce;
  published
    property OnError: TElectronBaseErrorEvent read FOnError write FOnError;
    property Active;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
  end;

implementation

{ TElectronNodeTLSSocketOptions }

constructor TElectronNodeTLSSocketOptions.Create;
begin
  FKeyPath := '';
  FMaxVersion := '';
  FCRLPath := '';
  FSSLProfile := '';
  FECDHCurve := '';
  FCertPath := '';
  FMinVersion := '';
  FCAPath := '';
  FPassphrase := '';
  FPFXPath := '';
  FClientCertEngine := '';
  FDHParam := '';
  FUseProfile := False;
  FSigalgs := '';
  FRejectUnauthorized := True;
end;

{ TElectronBaseConnection }

function TElectronBaseConnection.IsDatabaseNameStored: Boolean;
begin
  Result := FDatabaseName <> '';
end;

function TElectronBaseConnection.IsHostStored: Boolean;
begin
  Result := FHost <> 'localhost';
end;

function TElectronBaseConnection.IsPasswordStored: Boolean;
begin
  Result := FPassword <> '';
end;

function TElectronBaseConnection.IsUserStored: Boolean;
begin
  Result := FUser <> '';
end;

{ TElectronBaseClientDataSet }

constructor TElectronBaseClientDataSet.Create(AOwner: TComponent);
begin
  inherited;
  FIndexName := '';
  FTableName := '';
end;

function TElectronBaseClientDataSet.IsIndexNameStored: Boolean;
begin
  Result := FIndexName <> '';
end;

function TElectronBaseClientDataSet.IsTableNameStored: Boolean;
begin
  Result := FTableName <> '';
end;

procedure TElectronBaseClientDataSet.Refresh(Force: Boolean);
begin

end;

end.
