{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.FaunaDbCDS;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, DB, DBClient, WEBLib.Controls, Web, jsdelphisystem, js;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, DB, WEBLib.Controls, Web, jsdelphisystem;
  {$ENDIF}

type
  TFaunaDbErrorEvent = procedure(Sender: TObject; errorName, errorMsg: string) of object;
  TFaunaDbSignupUserExistsEvent = procedure(Sender: TObject; UserName: string) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  [JSLibReference('http://cdn.jsdelivr.net/gh/fauna/faunadb-js-release@latest/faunadb-min.js')]
  {$ENDIF}
  TWebFaunaDbClientDataset = class(TDataSet)
  private
    FClientKey, FCollectionName: string;
    FKeyFieldName, FUsersCollectionName, FUsersIndexName: string;
    FOnError: TFaunaDbErrorEvent;
    FOnUserExists: TFaunaDbSignupUserExistsEvent;
    FParams: TParams;
    procedure SetParams(const Value: TParams);
  protected
    procedure InternalClose; override;
    procedure InternalHandleException; override;
    procedure InternalInitFieldDefs; override;
    procedure InternalOpen; override;
    function IsCursorOpen: Boolean; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Login(aUserName, aPassword: String; IsSignUp: Boolean=False);
    procedure Refresh(Force: Boolean=False); reintroduce;
    procedure AddSortFieldDef(aField: String; isAscending: Boolean);
    procedure ClearSortFieldDefs;
  published
    property ClientKey: String read FClientKey write FClientKey;
    property CollectionName: String read FCollectionName write FCollectionName;
    property KeyFieldName: String read FKeyFieldName write FKeyFieldName;
    property Params: TParams read FParams write SetParams;
    property UsersCollectionName: String read FUsersCollectionName write FUsersCollectionName;
    property UsersIndexName: String read FUsersIndexName write FUsersIndexName;
    property OnError: TFaunaDbErrorEvent read FOnError write FOnError;
    property OnUserExists: TFaunaDbSignupUserExistsEvent read FOnUserExists write FOnUserExists;

    // Other
    property Active;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
  end;

implementation

procedure TWebFaunaDbClientDataset.InternalClose;
begin
  //
end;

procedure TWebFaunaDbClientDataset.InternalHandleException;
begin
  //
end;

procedure TWebFaunaDbClientDataset.InternalInitFieldDefs;
begin
  //
end;

procedure TWebFaunaDbClientDataset.InternalOpen;
begin
  //
end;

function TWebFaunaDbClientDataset.IsCursorOpen: Boolean;
begin
  Result := false;
end;

constructor TWebFaunaDbClientDataset.Create(AOwner: TComponent);
begin
  inherited;
  FClientKey := '';
  FCollectionName := ''; 
  FKeyFieldName := '';
  FUsersCollectionName := 'users';
  FUsersIndexName := 'users_by_email';
  FParams := TParams.Create(Self);
end;

destructor TWebFaunaDbClientDataset.Destroy;
begin
  FParams.Free;
  inherited;
end;

procedure TWebFaunaDbClientDataset.Login(aUserName, aPassword: String; IsSignUp: Boolean=False);
begin
end;

procedure TWebFaunaDbClientDataset.Refresh;
begin
end;

procedure TWebFaunaDbClientDataset.SetParams(const Value: TParams);
begin
  FParams.Assign(Value);
end;

procedure TWebFaunaDbClientDataset.AddSortFieldDef(aField: String; isAscending: Boolean);
begin
end;

procedure TWebFaunaDbClientDataset.ClearSortFieldDefs;
begin
end;


end.
