{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.JSON;

interface

uses
  Classes, Web, JS, SysUtils;

type
  TJSONObject = class;
  TJSONString = class;
  JSValue = variant;

  TJSONAncestor = class(TObject)
  private
  protected
    function GetStrValue: string; virtual;
    procedure SetStrValue(const Value: string); virtual;
  public
    property Value: string read GetStrValue write SetStrValue;
  end;

  TJSONValue = class(TJSONAncestor)
  private
    fjv: JSValue;
  protected
    property jv: JSValue read fjv write fjv;
  public
    constructor Create(AJSValue: JSValue); overload;
  end;

  TJSONPair = class(TObject)
  private
    fjv: TJSONValue;
    fjs: TJSONString;
  public
    constructor Create; overload;
    constructor Create(const Str: string; const Value: TJSONValue); overload;
    constructor Create(const Str: string; const Value: string); overload;
    destructor Destroy; override;
    property JsonValue: TJSONValue read fjv write fjv;
    property JsonString: TJSONString read fjs write fjs;
  end;


  TJSONObject = class(TJSONValue)
  private
    fjo: TJSObject;
  protected
    property jo: TJSObject read fjo write fjo;
  public
    constructor Create(AObject: TJSObject); overload;
    property JSObject: TJSObject read fjo;
    class function ParseJSONValue(const data: string): TJSONValue;
    function GetJSONValue(Name: string): string;
    function Get(Name: string): TJSONPair;
    function GetValue(const Name: string): TJSONValue;
    function AddPair(const Pair: TJSONPair): TJSONObject; overload;
    function AddPair(const Str: string; const Val: TJSONValue): TJSONObject; overload;
    function AddPair(const Str: string; const Val: string): TJSONObject; overload;
    function AddPair(const Str: string; const Val: Integer): TJSONObject; overload;
    function ToJSON: string;
  end;

  TJSONString = class(TJSONValue)
  private
    FValue: string;
  protected
    function GetStrValue: string; override;
    procedure SetStrValue(const Value: string); override;
  public

  end;

  TJSONNumber = class(TJSONString)
  private
    FInt: integer;
    FDouble: double;
  protected
    function GetStrValue: string; override;
    procedure SetStrValue(const Value: string); override;
  public
    constructor Create(ANumber: integer); overload;
    constructor Create(ANumber: double); overload;

    property AsInt: integer read FInt write FInt;
    property AsDouble: double read FDouble write FDouble;
  end;

  TJSONTrue = class(TJSONValue)
  protected
    function GetStrValue: string; override;
  end;

  TJSONFalse = class(TJSONValue)
  protected
    function GetStrValue: string; override;
  end;


  TJSONArray = class(TJSONObject)
  private
    fja: TJSArray;
    function GetItem(index: integer): TJSONObject;
    procedure SetItem(index: integer; const Value: TJSONObject);
    function GetCount: integer;
  protected
    property ja: TJSArray read fja write fja;
  public
    constructor Create(AArray: TJSArray); overload;
    procedure Add(const Element: string); overload;
    procedure Add(const Element: boolean); overload;
    procedure Add(const Element: integer); overload;
    procedure Add(const Element: double); overload;
    procedure Add(const Element: TJSONObject); overload;
    procedure Add(const Element: TJSONPair); overload;
    property Count: integer read GetCount;
    property Items[index: integer]: TJSONObject read GetItem write SetItem; default;
  end;

  TJSON = class(TObject)
    function Parse(s: string): TJSONObject;
  end;


implementation


function JSONObjectToString(const v: JSValue): string;
begin
  Result := '';
end;

{ TJSON }

function TJSON.Parse(s: string): TJSONObject;
begin
  Result := nil;
end;

{ TJSONArray }

constructor TJSONArray.Create(AArray: TJSArray);
begin
  inherited Create;
  ja := AArray;
end;

procedure TJSONArray.Add(const Element: integer);
begin
end;

procedure TJSONArray.Add(const Element: boolean);
begin
end;

procedure TJSONArray.Add(const Element: string);
begin
end;

procedure TJSONArray.Add(const Element: TJSONPair);
begin
end;

procedure TJSONArray.Add(const Element: TJSONObject);
begin
end;

procedure TJSONArray.Add(const Element: double);
begin
end;

function TJSONArray.GetCount: integer;
begin
  Result := ja.Length;
end;

function TJSONArray.GetItem(index: integer): TJSONObject;
begin
  Result := nil;
end;

procedure TJSONArray.SetItem(index: integer; const Value: TJSONObject);
begin
end;

{ TJSONObject }

function TJSONObject.AddPair(const Pair: TJSONPair): TJSONObject;
begin
  Result := Self;
end;

function TJSONObject.AddPair(const Str: string;
  const Val: TJSONValue): TJSONObject;
begin
  Result := Self;
end;

function TJSONObject.AddPair(const Str, Val: string): TJSONObject;
begin
  Result := Self;
end;

function TJSONObject.AddPair(const Str: string; const Val: Integer): TJSONObject;
begin
  Result := Self;
end;

constructor TJSONObject.Create(AObject: TJSObject);
begin
  inherited Create;
end;

function TJSONObject.Get(Name: string): TJSONPair;
begin
  Result := nil;
end;

function TJSONObject.GetJSONValue(Name: string): string;
begin
end;

function TJSONObject.GetValue(const Name: string): TJSONValue;
begin
  Result := nil;
end;

class function TJSONObject.ParseJSONValue(const data: string): TJSONValue;
begin
  Result := nil;
end;

function TJSONObject.ToJSON: string;
begin
  Result := '';
end;

{ TJSONPair }

constructor TJSONPair.Create;
begin
  inherited;
  fjs := TJSONString.Create;
end;

constructor TJSONPair.Create(const Str: string; const Value: TJSONValue);
begin
  inherited Create;
end;

constructor TJSONPair.Create(const Str, Value: string);
begin
  inherited Create;
end;

destructor TJSONPair.Destroy;
begin
  fjs.Free;
  inherited;
end;

{ TJSONAncestor }

function TJSONAncestor.GetStrValue: string;
begin
  Result := '';
end;

procedure TJSONAncestor.SetStrValue(const Value: string);
begin
  //
end;

{ TJSONString }

function TJSONString.GetStrValue: string;
begin
  Result := FValue;
end;

procedure TJSONString.SetStrValue(const Value: string);
begin
  FValue := Value;
end;

{ TJSONNumber }

constructor TJSONNumber.Create(ANumber: integer);
begin
  inherited Create;
end;

constructor TJSONNumber.Create(ANumber: double);
begin
  inherited Create;
end;

function TJSONNumber.GetStrValue: string;
begin
  Result := IntToStr(FInt);
end;

procedure TJSONNumber.SetStrValue(const Value: string);
begin
  FInt := StrToInt(Value);
end;

{ TJSONTrue }

function TJSONTrue.GetStrValue: string;
begin
   Result := 'true';
end;

{ TJSONFalse }

function TJSONFalse.GetStrValue: string;
begin
  Result := 'false';
end;


{ TJSONValue }

constructor TJSONValue.Create(AJSValue: JSValue);
begin
  inherited Create;
  jv := AJSValue;
end;

end.
