{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.REST;

interface

uses
   Classes, WEBLib.Controls, WEBLib.Storage, Web;

type
  THTTPResponseEvent = procedure(Sender: TObject; AResponse: string) of object;
  THTTPRequestResponseEvent = procedure(Sender: TObject; ARequest: TJSXMLHttpRequestRecord; AResponse: string) of object;
  THTTPAbortEvent = procedure(Sender: TObject) of object;
  THTTPErrorEvent = procedure(Sender: TObject; ARequest: TJSXMLHttpRequestRecord; Event: TJSEventRecord; var Handled: boolean) of object;
  THTTPCommand = (httpGET, httpPOST, httpPUT, httpDELETE, httpHEAD, httpCUSTOM);

  {$IFNDEF LCLTMSWEB}
  THTTPResponseProc = reference to procedure(AResponse: string; ARequest: TJSXMLHttpRequest);
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  THTTPResponseProc = procedure(AResponse: string; ARequest: TJSXMLHttpRequest);
  {$ENDIF}

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebHttpRequest = class(TComponent)
  private
    FURL: string;
    FOnResponse: THTTPResponseEvent;
    FOnAbort: THTTPAbortEvent;
    FHeaders: TStringList;
    FCommand: THTTPCommand;
    FCustomCommand: string;
    FPostData: string;
    FOnRequestResponse: THttpRequestResponseEvent;
    FPassword: string;
    FUser: string;
    FTimeout: integer;
    FOnTimeout: TNotifyEvent;
    FOnError: THTTPErrorEvent;
    procedure SetHeaders(const Value: TStringList);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute(AResponse: THTTPResponseProc = nil);
  published
    property Command: THTTPCommand read FCommand write FCommand default httpGET;
    property CustomCommand: string read FCustomCommand write FCustomCommand;
    property Headers: TStringList read FHeaders write SetHeaders;
    property Password: string read FPassword write FPassword;
    property PostData: string read FPostData write FPostData;
    property Timeout: integer read FTimeout write FTimeout default 0;
    property URL: string read FURL write FURL;
    property User: string read FUser write FUser;
    property OnAbort: THTTPAbortEvent read FOnAbort write FOnAbort;
    property OnError: THTTPErrorEvent read FOnError write FOnError;
    property OnResponse: THTTPResponseEvent read FOnResponse write FOnResponse;
    property OnRequestResponse: THttpRequestResponseEvent read FOnRequestResponse write FOnRequestResponse;
    property OnTimeout: TNotifyEvent read FOnTimeout write FOnTimeout;
  end;

  THttpResponse = procedure(Sender: TObject; AResponse: string) of object;

  TPersistTokens = class(TPersistent)
  private
    FKey: string;
    FEnabled: boolean;
  public
    constructor Create; reintroduce;
    procedure Assign(Source: TPersistent); override;
  published
    property Key: string read FKey write FKey;
    property Enabled: boolean read FEnabled write FEnabled default false;
  end;

  TRESTApp = class(TPersistent)
  private
    FKey: string;
    FSecret: string;
    FCallbackURL: string;
  public
    procedure Assign(Source: TPersistent); override;
  published
    property Key: string read FKey write FKey;
    property Secret: string read FSecret write FSecret;
    property CallbackURL: string read FCallbackURL write FCallbackURL;
  end;

  TCoreCloudHeader = record
    header: String;
    value: String;
  end;

  TCoreCloudHeaders = array of TCoreCloudHeader;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebRESTClient = class(TComponent)
  private
    FAccessToken: string;
    FOnAccessToken: TNotifyEvent;
    FOnResponse: THttpResponse;
    FPersistTokens: TPersistTokens;
    FApp: TRESTApp;
    FScopes: TStrings;
    FLocale: string;
    FOnRequestResponse: THttpRequestResponseEvent;
    FOnError: THttpErrorEvent;
    procedure SetPersistTokens(const Value: TPersistTokens);
    procedure SetApp(const Value: TRESTApp);
    procedure SetScopes(const Value: TStrings);
  protected
    property Locale: string read FLocale write FLocale;
    property Token_Access: string read FAccessToken;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure HttpsDelete(URL: string); virtual;
    procedure HttpsGet(URL: string); overload; virtual;
    procedure HttpsGet(URL: string; headers: TCoreCloudHeaders); overload; virtual;
    procedure HttpsPost(URL: string; Data: string); overload; virtual;
    procedure HttpsPost(URL: string; ContentType, Data: string); overload; virtual;
    procedure HttpsPut(URL: string; ContentType, Data: string); virtual;
    property AccessToken: string read FAccessToken write FAccessToken;
    function URLEncode(URI: string): string;
    function DateTimeToWL(dt: TDateTime): string;
    procedure Connect;
    procedure TestTokens; virtual;
    procedure ClearTokens; virtual;
  published
    property App: TRESTApp read FApp write SetApp;
    property PersistTokens: TPersistTokens read FPersistTokens write SetPersistTokens;
    property Scopes: TStrings read FScopes write SetScopes;

    property OnAccessToken: TNotifyEvent read FOnAccessToken write FOnAccessToken;
    property OnError: THttpErrorEvent read FOnError write FOnError;
    property OnResponse: THttpResponse read FOnResponse write FOnResponse;
    property OnRequestResponse: THttpRequestResponseEvent read FOnRequestResponse write FOnRequestResponse;
  end;


  TJSONValue = class(TObject)

  end;

  TJSONObject = class(TObject)
  public
    function Get(Name: string): string;
  end;

  TJSONArray = class(TJSONObject)
  private
    function GetItem(index: integer): TJSONObject;
    procedure SetItem(index: integer; const Value: TJSONObject);
    function GetCount: integer;
  public
    property Count: integer read GetCount;
    property Items[index: integer]: TJSONObject read GetItem write SetItem;
  end;


  TJSON = class(TObject)
    function Parse(s: string): TJSONObject;
  end;

  ansistrig = string;

procedure AddHeader(var AHeaders: TCoreCloudHeaders; Header: String; Value: String);

implementation

procedure AddHeader(var AHeaders: TCoreCloudHeaders; Header: String; Value: String);
begin

end;


constructor TWebHttpRequest.Create(AOwner: TComponent);
var
  FDesignTime: boolean;
begin
  inherited;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in Owner.ComponentState) or (csLoading in Owner.ComponentState));

  FHeaders := TStringList.Create;
  FTimeout := 0;

  if FDesignTime then
  begin
    FHeaders.Add('Cache-Control=no-cache, no-store, must-revalidate');
  end;

  FCommand := httpGET;
end;

destructor TWebHttpRequest.Destroy;
begin
  FHeaders.Free;
  inherited;
end;

procedure TWebHttpRequest.Execute;
begin
//
end;


{ TJSONArray }

function TJSONArray.GetCount: integer;
begin
  Result := 0;
end;

function TJSONArray.GetItem(index: integer): TJSONObject;
begin
  Result := nil;
end;

procedure TJSONArray.SetItem(index: integer; const Value: TJSONObject);
begin

end;

{ TJSON }

function TJSON.Parse(s: string): TJSONObject;
begin
  Result := nil;
end;

{ TJSONObject }

function TJSONObject.Get(Name: string): string;
begin

end;

{ TWebRESTClient }

procedure TWebRESTClient.ClearTokens;
begin
end;

procedure TWebRESTClient.Connect;
begin
  //
end;

constructor TWebRESTClient.Create(AOwner: TComponent);
begin
  inherited;
  FPersistTokens := TPersistTokens.Create;
  FApp := TRESTApp.Create;
  FScopes := TStringList.Create;
end;

function TWebRESTClient.DateTimeToWL(dt: TDateTime): string;
begin
  //
end;

destructor TWebRESTClient.Destroy;
begin
  FApp.Free;
  FScopes.Free;
  FPersistTokens.Free;
  inherited;
end;

procedure TWebRESTClient.HttpsDelete(URL: string);
begin

end;

procedure TWebRESTClient.HttpsGet(URL: string; headers: TCoreCloudHeaders);
begin
  //
end;

procedure TWebRESTClient.HttpsGet(URL: string);
begin
  //
end;

procedure TWebRESTClient.HttpsPost(URL, ContentType, Data: string);
begin
  //
end;

procedure TWebRESTClient.HttpsPut(URL, ContentType, Data: string);
begin

end;

procedure TWebRESTClient.SetApp(const Value: TRESTApp);
begin
  FApp.Assign(Value);
end;

procedure TWebRESTClient.SetPersistTokens(const Value: TPersistTokens);
begin
  FPersistTokens.Assign(Value);
end;

procedure TWebRESTClient.SetScopes(const Value: TStrings);
begin
  FScopes.Assign(Value);
end;

procedure TWebRESTClient.TestTokens;
begin

end;

function TWebRESTClient.URLEncode(URI: string): string;
begin

end;

procedure TWebRESTClient.HttpsPost(URL, Data: string);
begin
  //
end;

{ TPersistTokens }

procedure TPersistTokens.Assign(Source: TPersistent);
begin
  if (Source is TPersistTokens) then
  begin
    FKey := (Source as TPersistTokens).Key;
    FEnabled := (Source as TPersistTokens).Enabled;
  end;
end;

constructor TPersistTokens.Create;
begin
  inherited;
  FEnabled := false;
  FKey :=  '';
end;

{ TRESTApp }

procedure TRESTApp.Assign(Source: TPersistent);
begin
  if (Source is TRESTApp) then
  begin
    FKey := (Source as TRESTApp).Key;
    FSecret := (Source as TRESTApp).Secret;
    FCallbackURL := (Source as TRESTApp).CallbackURL;
  end;
end;

procedure TWebHttpRequest.SetHeaders(const Value: TStringList);
begin
  FHeaders.Assign(Value);
end;

end.
