{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}
unit WEBLib.SignIn;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, WinApi.Windows, System.SysUtils, Vcl.Controls, Vcl.Graphics,
  Types, WEBLib.Controls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, LCLType, LCLIntf, SysUtils, Controls, Graphics, Types, WEBLib.Controls;
  {$ENDIF}

type
  TGoogleSignedInEventArgs = class(TPersistent)
  private
    FEMail: string;
    FLastName: string;
    FImageUrl: string;
    FID: string;
    FToken: string;
    FFirstName: string;
  public
    property Token: string read FToken write FToken;
    property ID: string read FID write FID;
    property FirstName: string read FFirstName write FFirstName;
    property LastName: string read FLastName write FLastName;
    property ImageUrl: string read FImageUrl write FImageUrl;
    property EMail: string read FEMail write FEMail;
  end;

  TGoogleSignedInEvent = procedure(Sender: TObject; Args: TGoogleSignedInEventArgs) of object;

  TFacebookSignedInEventArgs = class(TPersistent)
  private
    FID: string;
    FName: string;
    FEmail: string;
  public
    property ID: string read FID write FID;
    property Name: string read FName write FName;
    property Email: string read FEmail write FEmail;
  end;

  TFacebookSignedInEvent = procedure(Sender: TObject; Args: TFacebookSignedInEventArgs) of object;

  TSignInServiceType = (stGoogle, stFacebook);

  TSignInService = class(TPersistent)
  private
    FVisible: Boolean;
    FAppKey: string;
    FEnabled: Boolean;
    FControlID: string;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Enabled: Boolean read FEnabled write FEnabled default True;
    property Visible: Boolean read FVisible write FVisible default True;
    property AppKey: string read FAppKey write FAppKey;
    property ControlID: string read FControlID write FControlID;
  end;

  TSignInServices = class(TPersistent)
  private
    FGoogle: TSignInService;
    FFacebook: TSignInService;
  public
    constructor Create; reintroduce;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Google: TSignInService read FGoogle write FGoogle;
    property Facebook: TSignInService read FFacebook write FFacebook;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSignIn = class(TWebCustomControl)
  private
    FOnGoogleSignedIn: TGoogleSignedInEvent;
    FOnGoogleSignedOut: TNotifyEvent;
    FOnFacebookSignedOut: TNotifyEvent;
    FOnFacebookSignedIn: TFacebookSignedInEvent;
    FServices: TSignInServices;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure SignIn(Service: TSignInServiceType);
    procedure SignOut(Service: TSignInServiceType);
  published
    property ElementClassName;
    property ElementID;
    property Enabled;

    property Services: TSignInServices read FServices write FServices;

    property OnGoogleSignedIn: TGoogleSignedInEvent read FOnGoogleSignedIn write FOnGoogleSignedIn;
    property OnGoogleSignedOut: TNotifyEvent read FOnGoogleSignedOut write FOnGoogleSignedOut;
    property OnFacebookSignedIn: TFacebookSignedInEvent read FOnFacebookSignedIn write FOnFacebookSignedIn;
    property OnFacebookSignedOut: TNotifyEvent read FOnFacebookSignedOut write FOnFacebookSignedOut;
  end;


implementation

{ TWebSignIn }

constructor TWebSignIn.Create(AOwner: TComponent);
begin
  inherited;
  FServices := TSignInServices.Create;
end;

destructor TWebSignIn.Destroy;
begin
  FServices.Free;
  inherited;
end;

procedure TWebSignIn.Paint;
var
  lRect: TRect;
  lBuffer: string;
begin
  Canvas.Brush.Color := Color;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  Canvas.Font.Style := [fsBold];
  lRect := ClientRect;
  Canvas.Rectangle(lRect);

  lBuffer := '';

  DrawText(Canvas.Handle, PChar(lBuffer), Length(lBuffer), lRect, DT_SINGLELINE or DT_END_ELLIPSIS or DT_CENTER or DT_VCENTER);
  Canvas.TextOut(4, 4, Name);
end;

procedure TWebSignIn.SignIn(Service: TSignInServiceType);
begin

end;

procedure TWebSignIn.SignOut(Service: TSignInServiceType);
begin

end;

{ TSignInService }

procedure TSignInService.Assign(Source: TPersistent);
begin
  inherited;
  if Source is TSignInService then
  begin
    FEnabled := (Source as TSignInService).Enabled;
    FVisible := (Source as TSignInService).Visible;
    FAppKey := (Source as TSignInService).AppKey;
    FControlID := (Source as TSignInService).ControlID;
  end;
end;

constructor TSignInService.Create;
begin
  FEnabled := True;
  FVisible := True;
  FAppKey := '';
  FControlID := '';
end;

destructor TSignInService.Destroy;
begin
  inherited;
end;

{ TSignInServices }

procedure TSignInServices.Assign(Source: TPersistent);
begin
  inherited;
  if Source is TSignInServices then
  begin
    FGoogle.Assign((Source as TSignInServices).Google);
    FFacebook.Assign((Source as TSignInServices).Facebook);
  end;
end;

constructor TSignInServices.Create;
begin
  FGoogle := TSignInService.Create;
  FFacebook := TSignInService.Create;
end;

destructor TSignInServices.Destroy;
begin
  FGoogle.Free;
  FFacebook.Free;
  inherited;
end;

end.
