{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            Copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.StdCtrls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Winapi.Windows, Winapi.Messages, VCL.Controls, VCL.Forms, VCL.StdCtrls, SysUtils,
  VCL.Graphics, Types, Vcl.Themes, WEBLib.Controls, Web;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, SysUtils, LCLIntf, LCLType, Types, Messages, Controls, Forms,
  StdCtrls, Graphics, WEBLib.Controls, Web;
  {$ENDIF}

type
  TWebEditCharCase = (wecLowerCase, wecNormal, wecMixedCase, wecUpperCase);
  TWebCheckBoxState = (cbChecked, cbGrayed, cbUnchecked);
  TDateTimeKind = (dtkDate, dtkTime);
  TEllipsisPosition = (epEndEllipsis, epNone);
  //TTextLayout = (tlTop, tlCenter, tlBottom);
  TSysLinkType = (sltID, sltURL);

  TEditType = (weString, weFloat, weHex, weNumeric, weSignedFloat, weSignedNumeric, weSearch);

  THTMLType = (tLABELTAG, tSPAN, tDIV, tH1,tH2, tH3, tH4, tH5, tH6, tP);

  TJSHTMLInputElement = Integer;
  TJSHTMLSelectElement = Integer;

  TAutoCompletion = (acOff,acHonorificPrefix, aGivenName, acAdditionalName, acFamilyName, acHonorificSuffix,
    acNickName, acEmail, acUserName, acNewPassword, acCurrentPassword, acOrganizationTitle, acOrganization,
    acStreetAddress, acAddressLine1, acAddressLine2, acAddressLine3, acAddressLevel1, acAddressLevel2,
    acAddressLevel3, acAddressLevel4, acCountry, acCountryName, acPostalCode, acCCName, acCCGivenName,
    acCCAdditionalName, acCCFamilyName, acCCNumber, acExpiry, acExpiryMonth, acExpiryYear, ccCSC,
    acType, acTransactionCurrency, acTransactionAmount, acLanguage, acBirthday, acBirthdayDay,
    acBirthDayMonth, acBirthDayYear, acSex, acTelephone, acTelephoneCountryCode, acTelephoneNational,
    acTelephoneAreaCode, acTelephoneLocal, acTelephoneExtension, acIMPP, acURL, acPhoto);


  TLinkClickEvent = procedure(Sender: TObject; Link: string; LinkType: TSysLinkType) of object;

  TWebCustomLabel = class(TCustomLabel)
  strict private
    FElementClassName: TElementClassName;
    FElementID: TElementID;
    FRole: string;
  private
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FHTMLType: THTMLType;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FChildOrder: integer;
    FEventStopPropagation: TEventPropagation;
    FLinkTouchEvents: boolean;
    FElementLabelClassName: TElementClassName;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetColorEx: TColor;
    procedure SetColorEx(const Value: TColor);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    procedure SetFocus;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color: TColor read GetColorEx write SetColorEx default clWhite;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementLabelClassName: TElementClassName read FElementLabelClassName write FElementLabelClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property HTMLType: THTMLType read FHTMLType write FHTMLType default tLABELTAG;
    property LinkTouchEvents: boolean read FLinkTouchEvents write FLinkTouchEvents;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
    property Role: string read FRole write FRole;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebLabel = class(TWebCustomLabel)
  private
    FOnTouchMove: TTouchEvent;
    FOnTouchStart: TTouchEvent;
    FOnTouchEnd: TTouchEvent;
    FOnTouchCancel: TTouchEvent;
  published
    property Align;
    property Alignment;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize;
    property Caption;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementLabelClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    {$IFNDEF LCLTMSWEB}
    property EllipsisPosition;
    {$ENDIF}
    property Enabled;
    property FocusControl;
    property Font;
    property Height;
    property HeightStyle;
    property HeightPercent;
    property Hint;
    property HTMLType;
    property Layout;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role;
    property ShowAccelChar;
    property ShowHint;
    property TextDirection;
    property Top;
    property Transparent default true;
    property Visible;
    property Width;
    property WordWrap;
    property WidthStyle;
    property WidthPercent;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnTouchStart: TTouchEvent read FOnTouchStart write FOnTouchStart;
    property OnTouchEnd: TTouchEvent read FOnTouchEnd write FOnTouchEnd;
    property OnTouchMove: TTouchEvent read FOnTouchMove write FOnTouchMove;
    property OnTouchCancel: TTouchEvent read FOnTouchCancel write FOnTouchCancel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TWebCustomInput = class(TWebCustomControl)
  private
    FEventStopPropagation: TEventPropagation;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
  end;

  TWebCustomEdit = class(TCustomEdit)
  strict private
    FElementClassName: TElementClassName;
    FElementID: TElementID;
  private
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FShowFocus: boolean;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FAutoCompletion: TAutoCompletion;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FEditType: TEditType;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FRequired: boolean;
    FAutoFocus: boolean;
    FPattern: string;
    FRole: string;
    FChildOrder: integer;
    FEventStopPropagation: TEventPropagation;
    FDesignTime: boolean;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetAutoSize: boolean;
    procedure SetAutoSizeEx(const Value: boolean);
  protected
    {$IFNDEF LCLTMSWEB}
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property AutoFocus: boolean read FAutoFocus write FAutoFocus default false;
    property AutoSize: boolean read GetAutoSize write SetAutoSizeEx default false;
    property AutoCompletion: TAutoCompletion read FAutoCompletion write FAutoCompletion default acOff;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property EditType: TEditType read FEditType write FEditType default weString;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
    property Pattern: string read FPattern write FPattern;
    property Required: boolean read FRequired write FRequired default false;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebEdit = class(TWebCustomEdit)
  private
    FCharCaseEx: TWebEditCharCase;
  public
    constructor Create(AOwner: TComponent); overload; override;
  published
    property Align;
    property Alignment;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoCompletion;
    property AutoFocus;
    property AutoSelect;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property CharCase: TWebEditCharCase read FCharCaseEx write FCharCaseEx default wecNormal;
    property ChildOrder;
    property Color;
    property DragMode;
    property EditType;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle;
    property HeightPercent;
    property HideSelection;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property MaxLength;
    property ParentFont;
    property Pattern;
    property PasswordChar;
    property ReadOnly;
    property Required;
    property Role;
    property ShowFocus;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property TextDirection;
    property TextHint;
    property Top;
    property Visible;
    property Width;
    property WidthStyle;
    property WidthPercent;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSpinEdit = class(TWebCustomInput)
  strict private
    FAutoSize:Boolean;
    FIncrement: Integer;
    FMaxValue: Integer;
    FMinValue: Integer;
    FText: string;
    FRole: string;
    FValue: Integer;
    FOnChange: TNotifyEvent;
    FDesignTime: boolean;
  private
    FShowFocus: boolean;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FTabOrder: integer;
    FReadOnly: boolean;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure SetValue(const Value: Integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property Text: string read FText write FText;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize:Boolean read FAutoSize write FAutoSize;
    property BiDiMode;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle;
    property HeightPercent;
    property Hint;
    property Increment: Integer read FIncrement write FIncrement;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property MaxValue: Integer read FMaxValue write FMaxValue;
    property MinValue: Integer read FMinValue write FMinValue;
    property ParentFont;
    property ReadOnly: boolean read FReadOnly write FReadOnly default false;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property Value: Integer read FValue write SetValue;
    property Visible;
    property Width;
    property WidthStyle;
    property WidthPercent;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebDateTimePicker = class(TWebCustomInput)
  strict private
    FDate: TDateTime;
    FKind: TDateTimeKind;
    FOnChange: TNotifyEvent;
    FText: string;
    FRole: string;
    FTime: TDateTime;
    FDesignTime: boolean;
  private
    FShowFocus: boolean;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FTabOrder: integer;
    FReadOnly: boolean;
    procedure SetDate(const Value: TDateTime);
    procedure SetKind(const Value: TDateTimeKind);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property DragMode;
    property Date: TDateTime read FDate write SetDate;
    property ElementClassName;
    property ElementID;
    property ElementFont;
    property ElementPosition;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle;
    property HeightPercent;
    property Hint;
    property Kind: TDateTimeKind read FKind write SetKind default dtkDate;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property ReadOnly: boolean read FReadOnly write FReadOnly default false;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property Text: string read FText write FText;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property Time: TDateTime read FTime write FTime;
    property Top;
    property Visible;
    property Width;
    property WidthStyle;
    property WidthPercent;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebButton = class(TCustomButton)
  strict private
    class constructor Create;
    class destructor Destroy;
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FButtonType: string;
    FRole: string;
    FChildOrder: integer;
    FDesignTime: boolean;
    FEventStopPropagation: TEventPropagation;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthStyle(const Value: TSizeStyle);
  protected
    {$IFNDEF LCLTMSWEB}
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BiDiMode;
    property ButtonType: string read FButtonType write FButtonType;
    property Color default clNone;
    property Caption;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property Top;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebCheckBox = class(TCustomCheckBox)
  strict private
    FState: TWebCheckBoxState;
    FDesignTime: boolean;
  protected
    procedure SetChecked(Value: Boolean); override;
    {$IFNDEF LCLTMSWEB}
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FShowFocus: boolean;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FChildOrder: integer;
    FRole: string;
    FEventStopPropagation: TEventPropagation;
    FElementButtonClassName: TElementClassName;
    FElementLabelClassName: TElementClassName;
    procedure SetState(const Value: TWebCheckBoxState);
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Caption;
    property Checked;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementLabelClassName: TElementClassName read FElementLabelClassName write FElementLabelClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property State: TWebCheckBoxState read FState write SetState;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property Top;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  TWebRadioButton = class(TCustomControl)
  {$ENDIF}
  {$IFNDEF LCLTMSWEB}
  TWebRadioButton = class(TButtonControl)
  {$ENDIF}
  strict private
    class constructor Create;
    class destructor Destroy;
  private
    FChecked: Boolean;
    FGroupName: string;
    FRole: string;
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FShowFocus: boolean;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FWidthStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FHeightStyle: TSizeStyle;
    FWidthPercent: TPercentSize;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FChildOrder: integer;
    FDesignTime: boolean;
    FEventStopPropagation: TEventPropagation;
    FElementButtonClassName: TElementClassName;
    FElementLabelClassName: TElementClassName;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
    {$IFDEF LCLTMSWEB}
    procedure SetCaption(const Value: TCaption);
    function GetCaption: TCaption;
    {$ENDIF}
  protected
    procedure SetChecked(Value: boolean); {$IFNDEF LCLTMSWEB}override;{$ENDIF}
    {$IFNDEF LCLTMSWEB}
    procedure CreateParams(var Params: TCreateParams); override;
    procedure UpdateStyleElements; override;
    procedure SetParent(AParent: TWinControl); override;
    {$ENDIF}
    {$IFDEF LCLTMSWEB}
    procedure Paint; override;
    {$ENDIF}
    procedure CreateWnd; override;
  public
    constructor Create(ID: string); reintroduce; overload; virtual;
    constructor Create(AOwner: TComponent); overload; override;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    {$IFDEF LCLTMSWEB}
    property Caption: TCaption read GetCaption write SetCaption;
    {$ENDIF}
    {$IFNDEF LCLTMSWEB}
    property Caption;
    {$ENDIF}
    property Checked: Boolean read FChecked write SetChecked;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementLabelClassName: TElementClassName read FElementLabelClassName write FElementLabelClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property GroupName: string read FGroupName write FGroupName;
    property Height;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebListBox = class(TCustomListBox)
  strict private
    FElementSelectHandle: TJSHTMLSelectElement;
    FOnChange: TNotifyEvent;
    FDesignTime: boolean;
  strict
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FTempItemIndex: integer;
  private
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FShowFocus: boolean;
    FRole: string;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FChildOrder: integer;
    FEventStopPropagation: TEventPropagation;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetItemIndexEx: integer;
    procedure SetItemIndexEx(const Value: integer);
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure Loaded; override;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementSelectHandle: TJSHTMLSelectElement read FElementSelectHandle write FElementSelectHandle;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BiDiMode;
    property BorderStyle;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property ItemHeight;
    property ItemIndex: integer read GetItemIndexEx write SetItemIndexEx;
    property Items;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property MultiSelect;
    property ParentColor;
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property Top;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TWebComboBoxStyle = (csDropDownList, csDropDown);

  TCustomWebComboBox = class(TCustomComboBox)
  strict private
    FElementSelectHandle:TJSHTMLSelectElement;
    FOnChange: TNotifyEvent;
    FDesignTime: boolean;
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FShowFocus: boolean;
    FRole: string;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FChildOrder: integer;
    FEventStopPropagation: TEventPropagation;
    FStyle: TWebComboBoxStyle;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
    function GetHeightEx: integer;
    procedure SetHeightEx(const Value: integer);
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementSelectHandle: TJSHTMLSelectElement read FElementSelectHandle write FElementSelectHandle;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
    property ItemIndex;
    property Items;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BiDiMode;
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height: integer read GetHeightEx write SetHeightEx;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property Left;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property Style: TWebComboBoxStyle read FStyle write FStyle default csDropDownList;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property Text;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    {$IFNDEF LCLTMSWEB}
    property TextHint;
    {$ENDIF}
    property Top;
    property Visible;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebComboBox = class(TCustomWebComboBox)
  published
    property ItemIndex;
    property Items;
  end;


  TCaretPosition = TPoint;

  TWebCustomMemo = class(TCustomMemo)
  strict private
    FElementInputHandle: TJSHTMLInputElement;
  private
    FChildOrder: integer;
  protected
    property ChildOrder: integer read FChildOrder write FChildOrder default 0;
  public
    property ElementInputHandle:TJSHTMLInputElement read FElementInputHandle write FElementInputHandle;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebMemo = class(TWebCustomMemo)
  private
    FElementID: TElementID;
    FElementClassName: TElementClassName;
    FWidthStyle: TSizeStyle;
    FHeightStyle: TSizeStyle;
    FHeightPercent: TPercentSize;
    FWidthPercent: TPercentSize;
    FShowFocus: boolean;
    FElementFont: TElementFont;
    FElementPosition: TElementPosition;
    FTextDirection: TTextDirection;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FTabOrder: integer;
    FRole: string;
    FEventStopPropagation: TEventPropagation;
    FDesignTime: boolean;
    procedure SetHeightPercent(const Value: TPercentSize);
    procedure SetHeightStyle(const Value: TSizeStyle);
    procedure SetWidthPercent(const Value: TPercentSize);
    procedure SetWidthStyle(const Value: TSizeStyle);
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    procedure BeginUpdate; virtual;
    procedure EndUpdate; virtual;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property EventStopPropagation: TEventPropagation read FEventStopPropagation write  FEventStopPropagation;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property AutoSize;
    property BiDiMode;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property DragMode;
    property ElementClassName: TElementClassName read FElementClassName write FElementClassName;
    property ElementID: TElementID read FElementID write FElementID;
    property ElementFont: TElementFont read FElementFont write FElementFont default efProperty;
    property ElementPosition: TElementPosition read FElementPosition write FElementPosition default epAbsolute;
    property Enabled;
    property Font;
    property Height;
    property HeightStyle: TSizeStyle read FHeightStyle write SetHeightStyle default ssAbsolute;
    property HeightPercent: TPercentSize read FHeightPercent write SetHeightPercent default 100;
    property Hint;
    property Left;
    property Lines;
    {$IFNDEF LCLTMSWEB}
    property Margins;
    {$ENDIF}
    property ParentColor;
    property ParentFont;
    property ReadOnly;
    property Role: string read FRole write FRole;
    property SelLength;
    property SelStart;
    property ShowFocus: boolean read FShowFocus write FShowFocus default true;
    property ShowHint;
    property TabOrder: integer read FTabOrder write FTabOrder default 0;
    property TabStop;
    property TextDirection: TTextDirection read FTextDirection write FTextDirection default tdDefault;
    property Top;
    property Visible;
    property WantTabs;
    property Width;
    property WidthStyle: TSizeStyle read FWidthStyle write SetWidthStyle default ssAbsolute;
    property WidthPercent: TPercentSize read FWidthPercent write SetWidthPercent default 100;
    property WordWrap;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebRadioGroup = class(TWebCustomControl)
  strict private
    FCaption: string;
    FRole: string;
    FColumns: Integer;
    FItemIndex: Integer;
    FItems: TStringList;
  private
    FOnChange: TNotifyEvent;
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
    FElementButtonClassName: TElementClassName;
    FElementLabelClassName: TElementClassName;
    FElementGroupClassName: TElementClassName;
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure SetItems(const Value: TStringList);
    procedure SetCaption(const Value: string);
    procedure SetColumns(const Value: Integer);
  protected
    procedure ItemsChanged(Sender: TObject);
    procedure Paint; override;
    procedure InitBootstrapStyle; override;
    procedure SetName(const Value: TComponentName); override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    destructor Destroy; override;
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Caption: string read FCaption write SetCaption;
    property ChildOrder;
    property Columns: Integer read FColumns write SetColumns;
    property DragMode;
    property ElementClassName;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementGroupClassName: TElementClassName read FElementGroupClassName write FElementGroupClassName;
    property ElementLabelClassName: TElementClassName read FElementLabelClassName write FElementLabelClassName;
    property ElementFont;
    property ElementPosition;
    property Font;
    property ItemIndex: Integer read FItemIndex write FItemIndex;
    property Items: TStringList read FItems write SetItems;
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowHint;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TCheckClickEvent = procedure(Sender: TObject; AIndex: integer) of object;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebCheckGroup = class(TWebCustomControl)
  private
    FElementButtonClassName: TElementClassName;
    FElementLabelClassName: TElementClassName;
    FItems: TStringList;
    FRole: string;
    FColumns: Integer;
    FCaption: string;
    FElementGroupClassName: TElementClassName;
    FOnCheckClick: TCheckClickEvent;
    procedure SetCaption(const Value: string);
    procedure SetColumns(const Value: Integer);
    procedure SetItems(const Value: TStringList);
    function GetChecked(AIndex: integer): boolean;
    procedure SetChecked(AIndex: integer; const Value: boolean);
  protected
    procedure ItemsChanged(Sender: TObject);
    procedure SetName(const Value: TComponentName); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); overload; override;
    constructor Create(AId: string); reintroduce; overload;
    destructor Destroy; override;
    property Checked[AIndex: integer]: boolean read GetChecked write SetChecked;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Caption: string read FCaption write SetCaption;
    property ChildOrder;
    property Columns: Integer read FColumns write SetColumns;
    property DragMode;
    property ElementClassName;
    property ElementButtonClassName: TElementClassName read FElementButtonClassName write FElementButtonClassName;
    property ElementGroupClassName: TElementClassName read FElementGroupClassName write FElementGroupClassName;
    property ElementLabelClassName: TElementClassName read FElementLabelClassName write FElementLabelClassName;
    property ElementFont;
    property ElementPosition;
    property Font;
    property Items: TStringList read FItems write SetItems;
    property ParentFont;
    property Role: string read FRole write FRole;
    property ShowHint;
    property Visible;
    property OnCheckClick: TCheckClickEvent read FOnCheckClick write FOnCheckClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;


  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebColorPicker = class(TWebCustomInput)
  strict private
    FColor: TColor;
    FOnSelect: TNotifyEvent;
    FRole: string;
  private
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
  public
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property Color: TColor read FColor write FColor;
    property DragMode;
    property Enabled;
    property Height;
    property Left;
    property Role: string read FRole write FRole;
    property Top;
    property Visible;
    property Width;
    property OnClick;
    property OnSelect: TNotifyEvent read FOnSelect write FOnSelect;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

  TWebScrollBarContent = class(TWebCustomControl)
  end;

  TWebScrollBarKind = (sbHorizontal, sbVertical);
  TWebScrollBarInc = Integer;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebScrollBar = class(TWebCustomControl)
  strict private
    FKind: TWebScrollBarKind;
    FLargeChange: TWebScrollBarInc;
    FMax: Integer;
    FRole: string;
    FMin: Integer;
    FOnChange: TNotifyEvent;
    FPageSize: Integer;
    FPosition: Integer;
    FSmallChange: TWebScrollBarInc;
  private
    FElementHandle: TJSHTMLElement;
    FParentElementID: TJSElement;
    FParentElement: TJSElement;
  public
    property ElementHandle: TJSHTMLElement read FElementHandle;
    property ParentElement: TJSElement read FParentElement write FParentElement;
    property ParentElementID: TJSElement read FParentElementID write FParentElementID;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property ChildOrder;
    property DragMode;
    property Height;
    property Kind: TWebScrollBarKind read FKind write FKind;
    property LargeChange: TWebScrollBarInc read FLargeChange write FLargeChange;
    property Left;
    property Max: Integer read FMax write FMax;
    property Min: Integer read FMin write FMin;
    property PageSize: Integer read FPageSize write FPageSize;
    property Position: Integer read FPosition write FPosition;
    property Role: string read FRole write FRole;
    property SmallChange: TWebScrollBarInc read FSmallChange write FSmallChange;
    property Top;
    property Visible;
    property Width;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

function GetAutoCompletionName(aAutoCompletion: TAutoCompletion): string;


implementation

uses
  WEBLib.Forms;

function GetAutoCompletionName(aAutoCompletion: TAutoCompletion): string;
begin
  //
end;


{ TWebRadioGroup }

procedure TWebRadioGroup.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
end;

constructor TWebRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TStringList.Create;
  FItems.OnChange := ItemsChanged;

  Width := 160;
  Height := 120;

  Columns := 1;
end;

constructor TWebRadioGroup.Create(AId: string);
begin
//
end;

destructor TWebRadioGroup.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

procedure TWebRadioGroup.InitBootstrapStyle;
begin
  inherited;
  ElementFont := efCSS;
  ElementClassName := 'custom-control custom-radio';
  ElementButtonClassName := 'custom-control-input';
  ElementLabelClassName := 'custom-control-label';
  ElementGroupClassName := 'modal-content';
end;

procedure TWebRadioGroup.ItemsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebRadioGroup.Paint;
var
  i,x,y,th,tc,col,row: integer;
  r: TRect;
begin
  r := ClientRect;
  Canvas.Brush.Color := clWindow;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clWindow;
  Canvas.Rectangle(r);

  Canvas.Font.Assign(Font);
  th := Round(Canvas.TextHeight('gh')* 1.4);
  tc := 0;

  if Caption <> '' then
  begin
    r.Top := r.Top + th div 2;
    tc := th;
  end;

  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clGray;
  Canvas.Rectangle(r);

  Canvas.Font.Assign(Font);
  Canvas.Pen.Style := psClear;
  Canvas.TextOut(4,0,Caption);

  col := 0;
  row := 0;

  for i := 0 to Items.Count - 1 do
  begin
    x := 4 + col * (Width div Columns);
    y := 2 + tc + row * th;

    r := Rect(x, y, x + 16, y + 16);
    if i = ItemIndex then
      DrawFrameControl(Canvas.Handle,r, DFC_BUTTON, DFCS_BUTTONRADIO or DFCS_CHECKED)
    else
      DrawFrameControl(Canvas.Handle,r, DFC_BUTTON, DFCS_BUTTONRADIO);

    Canvas.TextOut(x + 18, y, Items[i]);

    if col < FColumns - 1 then
      inc(col)
    else
    begin
      col := 0;
      inc(row);
    end;
  end;
end;

procedure TWebRadioGroup.SetCaption(const Value: string);
begin
  FCaption := Value;
  Invalidate;
end;

procedure TWebRadioGroup.SetColumns(const Value: Integer);
begin
  if (Value > 0) and (Value < 10) then
  begin
    FColumns := Value;
    Invalidate;
  end;
end;

procedure TWebRadioGroup.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
end;

procedure TWebRadioGroup.SetName(const Value: TComponentName);
begin
  if Caption = Name then
    Caption := Value;
  inherited;
end;

{ TWebButton }

class constructor TWebButton.Create;
begin
  {$IFNDEF LCLTMSWEB}
  TCustomStyleEngine.RegisterStyleHook(TWebButton, TButtonStyleHook);
  {$ENDIF}
end;

procedure TWebButton.BeginUpdate;
begin
  //
end;

constructor TWebButton.Create(AOwner: TComponent);
begin
  inherited;
  Width := 96;
  Height := 25;
  Color := clNone;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementClassName := 'btn';
    ElementFont := efCSS;
    HeightStyle := ssAuto;
  end;
end;

constructor TWebButton.Create(AId: string);
begin
//
end;

class destructor TWebButton.Destroy;
begin
  {$IFNDEF LCLTMSWEB}
  TCustomStyleEngine.UnRegisterStyleHook(TWebButton, TButtonStyleHook);
  {$ENDIF}
end;

procedure TWebButton.EndUpdate;
begin
  //
end;

procedure TWebButton.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebButton.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebButton.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;

    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}

procedure TWebButton.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebButton.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebRadioButton }

class constructor TWebRadioButton.Create;
begin
  {$IFNDEF LCLTMSWEB}
  TCustomStyleEngine.RegisterStyleHook(TWebRadioButton, TRadioButtonStyleHook);
  {$ENDIF}
end;

class destructor TWebRadioButton.Destroy;
begin
  {$IFNDEF LCLTMSWEB}
  TCustomStyleEngine.UnRegisterStyleHook(TWebRadioButton, TRadioButtonStyleHook);
  {$ENDIF}
end;

procedure TWebRadioButton.BeginUpdate;
begin
  //
end;

constructor TWebRadioButton.Create(ID: string);
begin
  Create(nil);
end;

constructor TWebRadioButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 113;
  Height := 22;
  Color := clNone;
  ControlStyle := [csSetCaption, csDoubleClicks];
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementFont := efProperty;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'custom-control custom-radio';
    ElementButtonClassName := 'custom-control-input';
    ElementLabelClassName := 'custom-control-label';
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebRadioButton.CreateParams(var Params: TCreateParams);
const
  Alignments: array[Boolean, TLeftRight] of DWORD =
    ((BS_LEFTTEXT, 0), (0, BS_LEFTTEXT));
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do
    Style := Style or BS_RADIOBUTTON;
end;
{$ENDIF}

procedure TWebRadioButton.CreateWnd;
begin
  inherited CreateWnd;
  {$IFNDEF LCLTMSWEB}
  SendMessage(Handle, BM_SETCHECK, WPARAM(FChecked), 0);
  {$ENDIF}
end;

procedure TWebRadioButton.EndUpdate;
begin
  //
end;

{$IFDEF LCLTMSWEB}
procedure TWebRadioButton.Paint;
begin
  Canvas.Pen.Color := clBlack;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Width := 1;
  Canvas.Brush.Style := bsClear;
  Canvas.Ellipse(0,0,16,16);
  if Checked then
  begin
    Canvas.Brush.Style := bsSolid;
    Canvas.Brush.Color := clBlack;
    Canvas.Ellipse(5,5,11,11);
  end;
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut(18,2,Caption);
end;
{$ENDIF}

procedure TWebRadioButton.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebRadioButton.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebRadioButton.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;
    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}

procedure TWebRadioButton.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebRadioButton.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebRadioButton.UpdateStyleElements;
begin
  Invalidate;
end;
{$ENDIF}

procedure TWebRadioButton.SetChecked(Value: boolean);
begin
  FChecked := Value;
  {$IFNDEF LCLTMSWEB}
  if HandleAllocated then
    SendMessage(Handle, BM_SETCHECK, WPARAM(Checked), 0);
  {$ENDIF}
  Invalidate;
end;

{$IFDEF LCLTMSWEB}
procedure TWebRadioButton.SetCaption(const Value: TCaption);
begin
  inherited Caption := Value;
  Invalidate;
end;

function TWebRadioButton.GetCaption: TCaption;
begin
  Result := inherited Caption;
end;

{$ENDIF}

{ TWebDateTimePicker }

constructor TWebDateTimePicker.Create(AOwner: TComponent);
begin
  inherited;
  Width := 170;
  Height := 22;
  Date := Now;
  WidthStyle := ssAbsolute;
  WidthPercent := 100;
  HeightStyle := ssAbsolute;
  HeightPercent := 100;
  FShowFocus := true;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'form-control';
  end;
end;

procedure TWebDateTimePicker.Paint;
var
  s: string;
  R: TRect;
begin
  inherited;

  if Kind = dtkDate then
    s := DateToStr(Date)
  else
    s := TimeToStr(Time);

  R := ClientRect;
  R.Left := R.Left + 4;

  Canvas.Font.Assign(Font);

  DrawText(Canvas.Handle, PChar(s), Length(s), R, DT_LEFT or DT_VCENTER or DT_SINGLELINE
      {$IFNDEF LCLTMSWEB}
      or DT_HIDEPREFIX
      {$ENDIF}
    );
end;

procedure TWebDateTimePicker.SetDate(const Value: TDateTime);
begin
  FDate := Value;
  Invalidate;
end;

procedure TWebDateTimePicker.SetKind(const Value: TDateTimeKind);
begin
  FKind := Value;
  Invalidate;
end;

{ TWebCheckBox }

procedure TWebCheckBox.BeginUpdate;
begin
  //
end;

constructor TWebCheckBox.Create(AOwner: TComponent);
begin
  inherited;
  Height := 22;
  Width := 113;
  Color := clNone;
  Checked := false;
  FState := cbUnchecked;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'custom-control custom-checkbox';
    ElementButtonClassName := 'custom-control-input';
    ElementLabelClassName := 'custom-control-label';
  end;
end;

constructor TWebCheckBox.Create(AId: string);
begin
//
end;

procedure TWebCheckBox.EndUpdate;
begin
  //
end;

procedure TWebCheckBox.SetChecked(Value: Boolean);
begin
  inherited;
  if Value then
    FState := cbChecked
  else
    FState := cbUnChecked;
end;

procedure TWebCheckBox.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCheckBox.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebCheckBox.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;
    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}

procedure TWebCheckBox.SetState(const Value: TWebCheckBoxState);
begin
  FState := Value;
  Checked := FState = cbChecked;
end;

procedure TWebCheckBox.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCheckBox.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebSpinEdit }

procedure TWebSpinEdit.CMFontChanged(var Message: TMessage);
begin
  if (csDesigning in ComponentState) and not (csReading in ComponentState) and not (csLoading in ComponentState) then
  begin
    Canvas.Font.Assign(Font);
    Height := Canvas.TextHeight('gh') + 6;
  end;
end;

constructor TWebSpinEdit.Create(AOwner: TComponent);
begin
  inherited;
  Width := 150;
  Height := 22;
  FMaxValue := 100;
  FIncrement := 1;
  WidthStyle := ssAbsolute;
  WidthPercent := 100;
  HeightStyle := ssAbsolute;
  HeightPercent := 100;
  FShowFocus := true;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'form-control';
  end;
end;

constructor TWebSpinEdit.Create(AId: string);
begin

end;

procedure TWebSpinEdit.Paint;
begin
  inherited;
  if ElementFont = efProperty then
    Canvas.Font.Assign(Font);

  Canvas.TextOut(4,4, IntToStr(Value));
end;

procedure TWebSpinEdit.SetValue(const Value: Integer);
begin
  FValue := Value;
  Invalidate;
end;

{ TWebEdit }

constructor TWebEdit.Create(AOwner: TComponent);
begin
  inherited;
  FCharCaseEx := wecNormal;
end;

{ TWebCustomLabel }

procedure TWebCustomLabel.BeginUpdate;
begin
  //
end;

constructor TWebCustomLabel.Create(AOwner: TComponent);
begin
  inherited;
  AutoSize := true;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  ShowAccelChar := True;
  Transparent := True;
end;

constructor TWebCustomLabel.Create(AId: string);
begin
  //
end;

procedure TWebCustomLabel.EndUpdate;
begin
  //
end;

function TWebCustomLabel.GetColorEx: TColor;
begin
  Result := inherited Color;
end;

procedure TWebCustomLabel.Paint;
var
  lRect: TRect;
  alignh,alignv,dtstyle: DWORD;
begin
  inherited;

  {$IFDEF LCLTMSWEB}
  if Color = clNone then
  begin
    if Assigned(Parent) and (Parent is TControl) then
      Canvas.Brush.Color := Parent.Color
    else
      Canvas.Brush.Color := clWhite;
  end
  else
    Canvas.Brush.Color := Color;

  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := Canvas.Brush.Color;
  Canvas.Pen.Style := psSolid;
  lRect := ClientRect;
  Canvas.Rectangle(lRect);
  {$ELSE}
  if not Transparent then
  begin
    Canvas.Brush.Color := Color;
    Canvas.Brush.Style := bsSolid;
    Canvas.Pen.Color := Color;
    Canvas.Pen.Style := psSolid;
    lRect := ClientRect;
    Canvas.Rectangle(lRect);
  end;
  {$ENDIF}

  alignh := 0;
  alignv := 0;

  case Alignment of
  taLeftJustify: alignh := DT_LEFT;
  taCenter: alignh := DT_CENTER;
  taRightJustify: alignh := DT_RIGHT;
  end;

  {$IFNDEF LCLTMSWEB}
  case Layout of
  Vcl.StdCtrls.tlTop: alignv := DT_TOP;
  Vcl.StdCtrls.tlCenter: alignv := DT_VCENTER;
  Vcl.StdCtrls.tlBottom: alignv := DT_BOTTOM;
  end;
  {$ENDIF}

  Canvas.Brush.Style := bsClear;

  dtstyle := DT_END_ELLIPSIS {$IFNDEF LCLTMSWEB}or DT_HIDEPREFIX{$ENDIF};

  if WordWrap or AutoSize then
    dtstyle := dtstyle or DT_WORDBREAK
  else
    dtstyle := dtstyle or DT_SINGLELINE or alignh or alignv;

  DrawText(Canvas.Handle, PChar(Caption), Length(Caption), lRect, dtstyle);
end;

procedure TWebCustomLabel.SetColorEx(const Value: TColor);
begin
  inherited Color := Value;

  if (csDesigning in ComponentState) and not (csLoading in ComponentState) and (Color <> clWhite) then
    Transparent := false;
end;

procedure TWebCustomLabel.SetFocus;
begin
  //Only a stub method
end;

procedure TWebCustomLabel.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomLabel.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomLabel.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomLabel.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebCustomEdit }

procedure TWebCustomEdit.BeginUpdate;
begin
  //
end;

constructor TWebCustomEdit.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  FEditType := weString;
  AutoSize := false;
  Height := 22;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'form-control';
  end;
end;

constructor TWebCustomEdit.Create(AId: string);
begin
//
end;

procedure TWebCustomEdit.EndUpdate;
begin
  //
end;

function TWebCustomEdit.GetAutoSize: boolean;
begin
  Result := inherited AutoSize;
end;

procedure TWebCustomEdit.SetAutoSizeEx(const Value: boolean);
begin
  inherited AutoSize := Value;
end;

procedure TWebCustomEdit.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomEdit.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{$IFNDEF LCLTMSWEB}
procedure TWebCustomEdit.SetParent(AParent: TWinControl);
begin
  inherited;

  if FDesignTime and Assigned(Parent) and not (csDestroying in ComponentState) then
  begin
    if IsRelativeParent(AParent) then
    begin
      ElementPosition := epIgnore;
      ElementFont := efCSS;
      WidthStyle := ssAuto;
      HeightStyle := ssAuto;
    end;

    ChildOrder := AParent.ControlCount - 1;
  end;
end;
{$ENDIF}

procedure TWebCustomEdit.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebCustomEdit.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebCustomInput }

constructor TWebCustomInput.Create(AOwner: TComponent);
begin
  inherited;
end;

constructor TWebCustomInput.Create(AId: string);
begin
//
end;

{ TWebListBox }

procedure TWebListBox.BeginUpdate;
begin
  //
end;

constructor TWebListBox.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'form-control';
  end;
end;

constructor TWebListBox.Create(AId: string);
begin
//
end;

procedure TWebListBox.EndUpdate;
begin
  //
end;

function TWebListBox.GetItemIndexEx: integer;
begin
  if csLoading in ComponentState then
    Result := FTempItemIndex
  else
    Result := inherited ItemIndex;
end;

procedure TWebListBox.Loaded;
begin
  inherited;
  ItemIndex := FTempItemIndex;
end;

procedure TWebListBox.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebListBox.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebListBox.SetItemIndexEx(const Value: integer);
begin
  if (csLoading in ComponentState) then
    FTempItemIndex := Value
  else
    inherited ItemIndex := Value;
end;

procedure TWebListBox.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebListBox.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TCustomWebComboBox }

procedure TCustomWebComboBox.BeginUpdate;
begin
  //
end;

constructor TCustomWebComboBox.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;
  //Style := TComboBoxStyle.csOwnerDrawVariable;
  Height := 22;
  FStyle := csDropDownList;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'custom-select';
  end;
end;

constructor TCustomWebComboBox.Create(AId: string);
begin
//
end;

procedure TCustomWebComboBox.EndUpdate;
begin
  //
end;

function TCustomWebComboBox.GetHeightEx: integer;
begin
  Result := inherited Height;
end;

procedure TCustomWebComboBox.SetHeightEx(const Value: integer);
begin
  ItemHeight := Value - 6;
  inherited Height := Value;
end;

procedure TCustomWebComboBox.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TCustomWebComboBox.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TCustomWebComboBox.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TCustomWebComboBox.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebMemo }

procedure TWebMemo.BeginUpdate;
begin
  //
end;

constructor TWebMemo.Create(AOwner: TComponent);
begin
  inherited;
  FWidthStyle := ssAbsolute;
  FWidthPercent := 100;
  FHeightStyle := ssAbsolute;
  FHeightPercent := 100;
  FShowFocus := true;
  FElementFont := efProperty;
  FElementPosition := epAbsolute;

  FDesignTime := (csDesigning in ComponentState) and not
                 ((csReading in AOwner.ComponentState) or (csLoading in AOwner.ComponentState));

  if FDesignTime and (GetOwnerCSSLibrary(AOwner) = cssBootstrap) then
  begin
    ElementFont := efCSS;
    ElementClassName := 'form-control';
  end;
end;

constructor TWebMemo.Create(AId: string);
begin
//
end;

procedure TWebMemo.EndUpdate;
begin
  //
end;

procedure TWebMemo.SetHeightPercent(const Value: TPercentSize);
begin
  if (FHeightPercent <> Value) then
  begin
    FHeightPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebMemo.SetHeightStyle(const Value: TSizeStyle);
begin
  if (FHeightStyle <> Value) then
  begin
    FHeightStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebMemo.SetWidthPercent(const Value: TPercentSize);
begin
  if (FWidthPercent <> Value) then
  begin
    FWidthPercent := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

procedure TWebMemo.SetWidthStyle(const Value: TSizeStyle);
begin
  if (FWidthStyle <> Value) then
  begin
    FWidthStyle := Value;
    if Assigned(Parent) then
      Parent.Realign;
  end;
end;

{ TWebProgressBar }


{ TWebCheckGroup }

constructor TWebCheckGroup.Create(AOwner: TComponent);
begin
  inherited;

  FItems := TStringList.Create;
  FItems.OnChange := ItemsChanged;

  Width := 160;
  Height := 120;

  Columns := 1;
end;

constructor TWebCheckGroup.Create(AId: string);
begin
  //
end;

destructor TWebCheckGroup.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TWebCheckGroup.GetChecked(AIndex: integer): boolean;
begin
  Result := true;
end;

procedure TWebCheckGroup.ItemsChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TWebCheckGroup.Paint;
var
  i,x,y,th,tc,col,row: integer;
  r: TRect;
begin
  r := ClientRect;

  Canvas.Brush.Color := clWindow;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clWindow;
  Canvas.Rectangle(r);

  Canvas.Font.Assign(Font);
  th := Round(Canvas.TextHeight('gh')* 1.4);
  tc := 0;

  if Caption <> '' then
  begin
    r.Top := r.Top + th div 2;
    tc := th;
  end;

  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clGray;
  Canvas.Rectangle(r);

  Canvas.Font.Assign(Font);
  Canvas.Pen.Style := psClear;
  Canvas.TextOut(4,0,Caption);

  col := 0;
  row := 0;

  for i := 0 to Items.Count - 1 do
  begin
    x := 4 + col * (Width div Columns);
    y := 2 + tc + row * th;

    r := Rect(x, y, x + 16, y + 16);
    DrawFrameControl(Canvas.Handle,r, DFC_BUTTON, DFCS_BUTTONCHECK);
    Canvas.TextOut(x + 18, y, Items[i]);

    if col < FColumns - 1 then
      inc(col)
    else
    begin
      col := 0;
      inc(row);
    end;
  end;
end;

procedure TWebCheckGroup.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

procedure TWebCheckGroup.SetChecked(AIndex: integer; const Value: boolean);
begin
  //
end;

procedure TWebCheckGroup.SetColumns(const Value: Integer);
begin
  if (Value > 0) and (Value < 10) then
  begin
    FColumns := Value;
    Invalidate;
  end;
end;

procedure TWebCheckGroup.SetItems(const Value: TStringList);
begin
  FItems.Assign(Value);
end;

procedure TWebCheckGroup.SetName(const Value: TComponentName);
begin
  if Caption = Name then
    Caption := Value;

  inherited;
end;

end.
