{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2019 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.SyntaxMemo;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Types, WEBLib.Controls;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Types, WEBLib.Controls;
  {$ENDIF}

type
  TSyntaxMemoMode = type string;
  TSyntaxMemoTheme = type string;

  TSyntaxMemoChangeCursorEvent = procedure(Sender: TObject; ARow: Integer; AColumn: Integer) of object;
  TSyntaxMemoChangeSelectionEvent = procedure(Sender: TObject; ASelectedText: string) of object;

  TSyntaxAutocomplete = (saNone, saBasic, saLive);
  TSyntaxWrapping = (swNone, swPrintMargin, swValue, swView);
  TSyntaxTextDirection = (stdLeftToRight, stdRightToLeft);

  TSyntaxCompletion = class(TCollectionItem)
  private
    FName: string;
    FMeta: string;
    FScore: Integer;
    FCaption: string;
    FValue: string;
  public
    constructor Create(ACollection: TCollection); override;
  published
    property Caption: string read FCaption write FCaption;
    property Meta: string read FMeta write FMeta;
    property Name: string read FName write FName;
    property Score: Integer read FScore write FScore default 0;
    property Value: string read FValue write FValue;
  end;

  TSyntaxCompleter = class(TCollection)
  private
    FOnChange: TNotifyEvent;
    procedure SetItem(Index: Integer; const Value: TSyntaxCompletion);
    function GetItem(Index: Integer): TSyntaxCompletion;
  protected
    procedure Changed; virtual;
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create;
    function Add: TSyntaxCompletion;
    function Insert(Index: Integer): TSyntaxCompletion;
    property Items[Index: integer]: TSyntaxCompletion read GetItem write SetItem;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebSyntaxMemo = class(TWebCustomControl)
  private
    FTheme: TSyntaxMemoTheme;
    FText: string;
    FCaretPosition: TPoint;
    FAutocompletion: TSyntaxAutocomplete;
    FCustomAutocomplete: TSyntaxCompleter;
    FFontSize: Integer;
    FFixedGutterWidth: Boolean;
    FFadeFoldWidgets: Boolean;
    FHighlightActiveLine: Boolean;
    FLines: TStringList;
    FMode: TSyntaxMemoMode;
    FPrintMargin: Integer;
    FSelLength: Integer;
    FReadOnly: Boolean;
    FSelStart: Integer;
    FShowGutter: Boolean;
    FShowPrintMargin: Boolean;
    FShowInvisibles: Boolean;
    FShowFoldWidgets: Boolean;
    FShowIndentGuides: Boolean;
    FShowLineNumbers: Boolean;
    FSoftTabs: Boolean;
    FTextDirection: TSyntaxTextDirection;
    FOnChangeCursor: TSyntaxMemoChangeCursorEvent;
    FWordWrapValue: Integer;
    FWordWrapIndent: Boolean;
    FOnChange: TNotifyEvent;
    FWordWrap: TSyntaxWrapping;
    FOnChangeSelection: TSyntaxMemoChangeSelectionEvent;
    FPersistentHorizontalScrollbar: Boolean;
    FPersistentVerticalScrollbar: Boolean;
    FTabSize: Integer;
    FFontName: string;
    procedure SetCustomAutocomplete(const Value: TSyntaxCompleter);
    procedure SetLines(const Value: TStringList);
  protected
    function GetDesignText: string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BeginUpdate; override;
    procedure Clear;
    procedure DisableLocalKeywords;
    procedure EndUpdate; override;
    procedure Find(AText: string);
    procedure FindAll(AText: string);
    procedure FindNext;
    procedure FindPrevious;
    procedure Focus;
    procedure InitializeKeyWords(ACompleter: TSyntaxCompleter);
    procedure InsertText(AText: string); overload;
    procedure InsertText(APosition: TPoint; AText: string); overload;
    procedure PreloadPascalKeywords;
    procedure Redo;
    procedure RemoveSelectedText;
    procedure RemoveCustomAutocompleter;
    procedure RemovePascalKeywords;
    procedure Replace(AReplacement: string); overload; //replace the selected text
    procedure Replace(AText, AReplacement: string); overload;
    procedure ReplaceAll(AReplacement: string); overload; //replace the selected text
    procedure ReplaceAll(AText, AReplacement: string); overload;
    procedure SelectAll;
    procedure Undo;
    procedure Unselect;
    function OpenSearchBox: Boolean;
    property CaretPosition: TPoint read FCaretPosition write FCaretPosition;
    property FontName: string read FFontName write FFontName;
    property Text: string read FText write FText;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property Autocompletion: TSyntaxAutocomplete read FAutocompletion write FAutocompletion default saBasic;
    property CustomAutocomplete: TSyntaxCompleter read FCustomAutocomplete write SetCustomAutocomplete;
    property FadeFoldWidgets: Boolean read FFadeFoldWidgets write FFadeFoldWidgets default False;
    property FixedGutterWidth: Boolean read FFixedGutterWidth write FFixedGutterWidth default False;
    property FontSize: Integer read FFontSize write FFontSize default 12;
    property Height;
    property HighlightActiveLine: Boolean read FHighlightActiveLine write FHighlightActiveLine default True;
    property Left;
    property Lines: TStringList read FLines write SetLines;
    property Mode: TSyntaxMemoMode read FMode write FMode;
    property PersistentHorizontalScrollbar: Boolean read FPersistentHorizontalScrollbar write FPersistentHorizontalScrollbar default False;
    property PersistentVerticalScrollbar: Boolean read FPersistentVerticalScrollbar write FPersistentVerticalScrollbar default False;
    property PrintMargin: Integer read FPrintMargin write FPrintMargin default 80;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property SelLength: Integer read FSelLength write FSelLength default 0;
    property SelStart: Integer read FSelStart write FSelStart default 0;
    property ShowFoldWidgets: Boolean read FShowFoldWidgets write FShowFoldWidgets default True;
    property ShowGutter: Boolean read FShowGutter write FShowGutter default True;
    property ShowIndentGuides: Boolean read FShowIndentGuides write FShowIndentGuides default True;
    property ShowInvisibles: Boolean read FShowInvisibles write FShowInvisibles default False;
    property ShowLineNumbers: Boolean read FShowLineNumbers write FShowLineNumbers default True;
    property ShowPrintMargin: Boolean read FShowPrintMargin write FShowPrintMargin default True;
    property SoftTabs: Boolean read FSoftTabs write FSoftTabs default True;
    property TextDirection: TSyntaxTextDirection read FTextDirection write FTextDirection default stdLeftToRight;
    property TabOrder;
    property TabSize: Integer read FTabSize write FTabSize default 2;
    property TabStop default true;
    property Theme: TSyntaxMemoTheme read FTheme write FTheme;
    property Top;
    property Visible;
    property Width;
    property WordWrap: TSyntaxWrapping read FWordWrap write FWordWrap default swNone;
    property WordWrapIndented: Boolean read FWordWrapIndent write FWordWrapIndent default True;
    property WordWrapValue: Integer read FWordWrapValue write FWordWrapValue default 80;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnChangeCursor: TSyntaxMemoChangeCursorEvent read FOnChangeCursor write FOnChangeCursor;
    property OnChangeSelection: TSyntaxMemoChangeSelectionEvent read FOnChangeSelection write FOnChangeSelection;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
  end;

implementation

{ TWebSyntaxMemo }

procedure TWebSyntaxMemo.BeginUpdate;
begin
  inherited;
end;

procedure TWebSyntaxMemo.Clear;
begin
  //
end;

constructor TWebSyntaxMemo.Create(AOwner: TComponent);
begin
  inherited;
  Width := 400;
  Height := 300;
  FCustomAutocomplete := TSyntaxCompleter.Create;
  FLines := TStringList.Create;
  TabStop := True;
  FAutocompletion := saBasic;
  FFadeFoldWidgets := False;
  FFixedGutterWidth := False;
  FFontSize := 12;
  FHighlightActiveLine := True;
  FPersistentHorizontalScrollbar := False;
  FPersistentVerticalScrollbar := False;
  FPrintMargin := 80;
  FReadOnly := False;
  FSelLength := 0;
  FSelStart := 0;
  FShowFoldWidgets := True;
  FShowGutter := True;
  FShowIndentGuides := True;
  FShowInvisibles := False;
  FShowLineNumbers := True;
  FShowPrintMargin := True;
  FSoftTabs := True;
  FTabSize := 2;
  FTextDirection := stdLeftToRight;
  FWordWrap := swNone;
  FWordWrapIndent := True;
  FWordWrapValue := 80;
end;

destructor TWebSyntaxMemo.Destroy;
begin
  FCustomAutocomplete.Free;
  FLines.Free;
  inherited;
end;

procedure TWebSyntaxMemo.DisableLocalKeywords;
begin
  //
end;

procedure TWebSyntaxMemo.EndUpdate;
begin
  inherited;
end;

procedure TWebSyntaxMemo.Find(AText: string);
begin
  //
end;

procedure TWebSyntaxMemo.FindAll(AText: string);
begin
  //
end;

procedure TWebSyntaxMemo.FindNext;
begin
  //
end;

procedure TWebSyntaxMemo.FindPrevious;
begin
  //
end;

procedure TWebSyntaxMemo.Focus;
begin
  //
end;

function TWebSyntaxMemo.GetDesignText: string;
begin
  Result := Name;
end;

procedure TWebSyntaxMemo.InitializeKeyWords(ACompleter: TSyntaxCompleter);
begin
  //
end;

procedure TWebSyntaxMemo.InsertText(AText: string);
begin
  //
end;

procedure TWebSyntaxMemo.InsertText(APosition: TPoint; AText: string);
begin
  //
end;

function TWebSyntaxMemo.OpenSearchBox: Boolean;
begin
  Result := True;
end;

procedure TWebSyntaxMemo.PreloadPascalKeywords;
begin
  //
end;

procedure TWebSyntaxMemo.Redo;
begin
  //
end;

procedure TWebSyntaxMemo.RemoveCustomAutocompleter;
begin
  //
end;

procedure TWebSyntaxMemo.RemovePascalKeywords;
begin
  //
end;

procedure TWebSyntaxMemo.RemoveSelectedText;
begin
  //
end;

procedure TWebSyntaxMemo.Replace(AText, AReplacement: string);
begin
  //
end;

procedure TWebSyntaxMemo.Replace(AReplacement: string);
begin
  //
end;

procedure TWebSyntaxMemo.ReplaceAll(AReplacement: string);
begin
  //
end;

procedure TWebSyntaxMemo.ReplaceAll(AText, AReplacement: string);
begin
  //
end;

procedure TWebSyntaxMemo.SelectAll;
begin
  //
end;

procedure TWebSyntaxMemo.SetCustomAutocomplete(const Value: TSyntaxCompleter);
begin
  FCustomAutocomplete.Assign(Value);
end;

procedure TWebSyntaxMemo.SetLines(const Value: TStringList);
begin
  FLines.Assign(Value);
end;

procedure TWebSyntaxMemo.Undo;
begin
  //
end;

procedure TWebSyntaxMemo.Unselect;
begin
  //
end;

{ TSyntaxCompletion }

constructor TSyntaxCompletion.Create(ACollection: TCollection);
begin
  inherited;
  FScore := 0;
end;

{ TSyntaxCompleter }

function TSyntaxCompleter.Add: TSyntaxCompletion;
begin
  Result := TSyntaxCompletion(inherited Add);
end;

procedure TSyntaxCompleter.Changed;
begin
  if Assigned(OnChange) then
    OnChange(Self);
end;

constructor TSyntaxCompleter.Create;
begin
  inherited Create(TSyntaxCompletion);
end;

function TSyntaxCompleter.GetItem(Index: Integer): TSyntaxCompletion;
begin
  Result := TSyntaxCompletion(inherited Items[Index]);
end;

function TSyntaxCompleter.Insert(Index: Integer): TSyntaxCompletion;
begin
  Result := TSyntaxCompletion(inherited Insert(Index));
end;

procedure TSyntaxCompleter.SetItem(Index: Integer;
  const Value: TSyntaxCompletion);
begin
  inherited Items[Index] := Value;
end;

procedure TSyntaxCompleter.Update(Item: TCollectionItem);
begin
  inherited;
  Changed;
end;

end.
