unit libfirestore;

interface

uses
  JS, Web;
  
type
  TFirestore = class;
  TFirestoreCollection = class;

  TFireBaseError = Class(TJSObject)
  private
    FMessage: String;
    FName: String;
  Public
    property message : String Read FMessage;
    property name : String Read FName;
  end;

  TFireBaseApp = class(TJSObject)
  end;

  TFireBaseAuthProvider = class(TJSObject)
  end;
  TFireBaseAuthUser = class(TJSObject)
  private
    fDisplayName: String; 
    fEmail: String; 
  public
    property displayName: String read fDisplayName;
    property email: String read fEmail;
  end;
  TFireBaseGoogleAuthProvider = class(TJSObject)
  public
    constructor New;
    procedure addScope(aScope: String);
  end;


  TFireBaseAuthStateObserver = reference to Procedure(aUser: TFireBaseAuthUser);
  TFireBaseJSErrorProc = reference to Procedure(anError: TFireBaseError);
  TFireBaseUnsubscribe = reference to Procedure;

  TFireBaseAuth = class(TJSObject)
  private
    fCurrentUser: TFireBaseAuthUser; 
  public
    function signInWithPopup(aProvider: TFireBaseGoogleAuthProvider): TJSPromise;
    function signOut: TJSPromise; 
    function onAuthStateChanged(observer: TFireBaseAuthStateObserver): TFireBaseUnsubscribe; overload;
    function onAuthStateChanged(observer: TFireBaseAuthStateObserver; errorProc: TFireBaseJSErrorProc): TFireBaseUnsubscribe; overload;
    property currentUser: TFireBaseAuthUser read fCurrentUser;
  end;

  TFireBase = class(TJSObject)
    class var apps: TJSArray;
    class function initializeApp(aConfig: TJSObject): TFireBaseApp;
    class function firestore: TFirestore;
    class function Auth: TFireBaseAuth;
  end;

  TFirestore = class (TJSObject)
    function collection(collectionPath: string): TFirestoreCollection;
  end;

  TFirestoreDocument = class(TJSObject)
    function doSet(aData: TJSObject): TJSPromise;
    function get(options: TJSObject): TJSPromise;
    function delete: TJSPromise;
  end;

  TFirestoreQuery = class(TJSPromise)
    function get(options: TJSObject): TJSPromise;
    function orderBy(aField: String): TFirestoreQuery; overload;
    function orderBy(aField: String; aDirection: String): TFirestoreQuery; overload;
  end;

  TFirestoreCollection = class(TJSObject)
    function add(data: TJSObject): TJSPromise;
    function get: TJSPromise; overload;
    function get(options: TJSObject): TJSPromise; overload;
    function doc(anId: String): TFirestoreDocument;
    function orderBy(aField: String): TFirestoreQuery; overload;
    function orderBy(aField: String; aDirection: String): TFirestoreQuery; overload;
  end;

  TFirestoreQueryDocumentSnapshot = class(TJSObject)
  private
    fId: String;
  public
    function data: TJSObject;
    property id: String read fId;
  end;

  TJSFirestoreQSEvent = reference to procedure(doc: TFirestoreQueryDocumentSnapshot);

  TFirestoreQuerySnapshot = class(TJSObject)
  private
   fdocs : TJSArray;
  public
    procedure forEach(aCallBack : TJSFirestoreQSEvent);
    property docs: TJSArray read fdocs;
  end;
  
 TFireBaseAuthUserCredential = class(TJSObject)
  private
    fUser: TFireBaseAuthUser; 
  public
    property user: TFireBaseAuthUser read fUser;
  end;

// FireBase App must be initialized before calling
function getLoggedInUser: TJSPromise;  

implementation
{$HINTS OFF}
{$WARNINGS OFF}

constructor TFireBaseGoogleAuthProvider.New;
begin
end;

procedure TFireBaseGoogleAuthProvider.addScope(aScope: String);
begin
end;

function TFireBaseAuth.signInWithPopup(aProvider: TFireBaseGoogleAuthProvider): TJSPromise;
begin
end;

function TFireBaseAuth.signOut: TJSPromise;
begin
end;

function TFireBaseAuth.onAuthStateChanged(observer: TFireBaseAuthStateObserver): TFireBaseUnsubscribe;
begin
end;

function TFireBaseAuth.onAuthStateChanged(observer: TFireBaseAuthStateObserver; errorProc: TFireBaseJSErrorProc): TFireBaseUnsubscribe;
begin
end;

class function TFireBase.Auth: TFireBaseAuth;
begin

end;

class function TFireBase.initializeApp(aConfig: TJSObject): TFireBaseApp;
begin
end;

class function TFireBase.firestore: TFirestore;
begin
end;

function TFirestore.collection(collectionPath: string): TFirestoreCollection;
begin
end;

function TFirestoreDocument.doSet(aData: TJSObject): TJSPromise;
begin
end;

function TFirestoreDocument.get(options: TJSObject): TJSPromise;
begin
end;

function TFirestoreDocument.delete: TJSPromise;
begin
end;

function TFirestoreQuery.get(options: TJSObject): TJSPromise;
begin
end;

function TFirestoreQuery.orderBy(aField: String): TFirestoreQuery;
begin
end;

function TFirestoreQuery.orderBy(aField: String; aDirection: String): TFirestoreQuery;
begin
end;

function TFirestoreCollection.add(data: TJSObject): TJSPromise;
begin
end;

function TFirestoreCollection.get: TJSPromise;
begin
end;

function TFirestoreCollection.get(options: TJSObject): TJSPromise;
begin
end;

function TFirestoreCollection.doc(anId: String): TFirestoreDocument;
begin
end;

function TFirestoreCollection.orderBy(aField: String): TFirestoreQuery;
begin
end;

function TFirestoreCollection.orderBy(aField: String; aDirection: String): TFirestoreQuery;
begin
end;

function TFirestoreQueryDocumentSnapshot.data: TJSObject;
begin
end;

procedure TFirestoreQuerySnapshot.forEach(aCallBack : TJSFirestoreQSEvent);
begin
end;

function getLoggedInUser: TJSPromise;
begin
end;

end.
