unit libserviceworkers;

{$IFDEF LCLTMSWEB}
{$MODE ObjFPC}
{$H+}
{$modeswitch externalclass}
{$ENDIF}

interface

uses
  SysUtils, JS, web, jsdelphisystem;

type
  // Forward class definitions
  TJSMessagePort = class;
  TJSAbortSignal = class;
  TJSHeaders = class;
  TJSBody = class;
  TJSRequest = class;
  TJSResponse = class;
  TJSAbstractWorker = class;
  TJSWorker = class;
  TJSSharedWorker = class;
  TJSServiceWorker = class;
  TJSServiceWorkerRegistration = class;
  TJSServiceWorkerContainer = class;
  TJSNavigationPreloadManager = class;
  TJSServiceWorkerGlobalScope = class;
  TJSClient = class;
  TJSWindowClient = class;
  TJSClients = class;
  TJSExtendableEvent = class;
  TJSFetchEvent = class;
  TJSExtendableMessageEvent = class;
  TJSCache = class;
  TJSCacheStorage = class;
  TJSPostMessageOptions = class;
  TJSRequestInit = class;
  TJSResponseInit = class;
  TJSWorkerOptions = class;
  TJSRegistrationOptions = class;
  TJSNavigationPreloadState = class;
  TJSClientQueryOptions = class;
  TJSExtendableEventInit = class;
  TJSFetchEventInit = class;
  TJSExtendableMessageEventInit = class;
  TJSCacheQueryOptions = class;
  TJSMultiCacheQueryOptions = class;
  ReferrerPolicy = string;
  RequestDestination = string;
  RequestMode = string;
  RequestCredentials = string;
  RequestCache = string;
  RequestRedirect = string;
  ResponseType = string;
  WorkerType = string;
  ServiceWorkerState = string;
  ServiceWorkerUpdateViaCache = string;
  FrameType = string;
  ClientType = string;
  // Union of sequence, record
  HeadersInit = JSValue; 
  // Union of Blob, BufferSource, FormData, URLSearchParams, ReadableStream, USVstring
  BodyInit = JSValue; 
  // Union of Request, USVstring
  RequestInfo = JSValue; 
  
  { --------------------------------------------------------------------
    TJSPostMessageOptions
    --------------------------------------------------------------------}
  
  TTJSObjectDynArray = array of TJSObject;
  TJSPostMessageOptions = class(TJSObject)
    transfer : TTJSObjectDynArray;
  end;
  
  { --------------------------------------------------------------------
    TJSRequestInit
    --------------------------------------------------------------------}
  
  TJSRequestInit = class(TJSObject)
    method : string;
    headers : HeadersInit;
    body : BodyInit;
    referrer : string;
    _referrerPolicy : ReferrerPolicy;
    mode : RequestMode;
    credentials : RequestCredentials;
    cache : RequestCache;
    redirect : RequestRedirect;
    integrity : string;
    keepalive : Boolean;
    signal : TJSAbortSignal;
    window : JSValue;
  end;
  
  { --------------------------------------------------------------------
    TJSResponseInit
    --------------------------------------------------------------------}
  
  TJSResponseInit = class(TJSObject)
    status : Cardinal;
    statusText : string;
    headers : HeadersInit;
  end;
  
  { --------------------------------------------------------------------
    TJSWorkerOptions
    --------------------------------------------------------------------}
  
  TJSWorkerOptions = class(TJSObject)
    type_ : WorkerType;
    credentials : RequestCredentials;
    name : string;
  end;
  
  { --------------------------------------------------------------------
    TJSRegistrationOptions
    --------------------------------------------------------------------}
  
  TJSRegistrationOptions = class(TJSObject)
    scope : string;
    type_ : WorkerType;
    updateViaCache : ServiceWorkerUpdateViaCache;
  end;
  
  { --------------------------------------------------------------------
    TJSNavigationPreloadState
    --------------------------------------------------------------------}
  
  TJSNavigationPreloadState = class(TJSObject)
    enabled : Boolean;
    headerValue : string;
  end;
  
  { --------------------------------------------------------------------
    TJSClientQueryOptions
    --------------------------------------------------------------------}
  
  TJSClientQueryOptions = class(TJSObject)
    includeUncontrolled : Boolean;
    type_ : ClientType;
  end;
  
  { --------------------------------------------------------------------
    TJSExtendableEventInit
    --------------------------------------------------------------------}
  
  TJSExtendableEventInit = class(TJSObject)
  end;
  
  { --------------------------------------------------------------------
    TJSFetchEventInit
    --------------------------------------------------------------------}
  
  TJSFetchEventInit = class(TJSObject)
    request : TJSRequest;
    preloadResponse : TJSPromise;
    clientId : string;
    resultingClientId : string;
    replacesClientId : string;
  end;
  
  { --------------------------------------------------------------------
    TJSExtendableMessageEventInit
    --------------------------------------------------------------------}
  
  TTJSMessagePortDynArray = array of TJSMessagePort;
  TJSExtendableMessageEventInit = class(TJSObject)
    data : JSValue;
    origin : string;
    lastEventId : string;
    source : JSValue;
    ports : TTJSMessagePortDynArray;
  end;
  
  { --------------------------------------------------------------------
    TJSCacheQueryOptions
    --------------------------------------------------------------------}
  
  TJSCacheQueryOptions = class(TJSObject)
    ignoreSearch : Boolean;
    ignoreMethod : Boolean;
    ignoreVary : Boolean;
  end;
  
  { --------------------------------------------------------------------
    TJSMultiCacheQueryOptions
    --------------------------------------------------------------------}
  
  TJSMultiCacheQueryOptions = class(TJSObject)
    cacheName : string;
  end;
  
  { --------------------------------------------------------------------
    TJSMessagePort
    --------------------------------------------------------------------}
  
  TJSMessagePort = class(TJSEventTarget)
  private
  public
    
      onmessage : TJSEventHandler;
      onmessageerror : TJSEventHandler;
    procedure postMessage(message : JSValue; transfer : TTJSObjectDynArray); overload;
    procedure postMessage(message : JSValue; options : TJSPostMessageOptions); overload;
    procedure postMessage(message : JSValue); overload;
    procedure start;
    procedure close;
  end;
  
  { --------------------------------------------------------------------
    TJSAbortSignal
    --------------------------------------------------------------------}
  
  TJSAbortSignal = class(TJSEventTarget)
  private
    Faborted : Boolean;
  public
      onabort : TJSEventHandler;
    property aborted : Boolean read Faborted;
  end;
  
  { --------------------------------------------------------------------
    TJSHeaders
    --------------------------------------------------------------------}
  
  TJSHeaders = class
  private
  public
    procedure append(name : string; value : string);
    procedure delete(name : string);
    function get(name : string): string;
    function has(name : string): Boolean;
    procedure set_(name : string; value : string);
  end;
  
  { --------------------------------------------------------------------
    TJSBody
    --------------------------------------------------------------------}
  
  TJSBody = class
  private
    Fbody : TJSObject;
    FbodyUsed : Boolean;
  public
    function arrayBuffer: TJSPromise;
    function blob: TJSPromise;
    function formData: TJSPromise;
    function json: TJSPromise;
    function text: TJSPromise;
    property body : TJSObject read Fbody;
    property bodyUsed : Boolean read FbodyUsed;
  end;
  
  { --------------------------------------------------------------------
    TJSRequest
    --------------------------------------------------------------------}
  
  TJSRequest = class(TJSBody)
  private
    Fmethod : string;
    Furl : string;
    Fheaders : TJSHeaders;
    Fdestination : RequestDestination;
    Freferrer : string;
    FreferrerPolicy : ReferrerPolicy;
    Fmode : RequestMode;
    Fcredentials : RequestCredentials;
    Fcache : RequestCache;
    Fredirect : RequestRedirect;
    Fintegrity : string;
    Fkeepalive : Boolean;
    FisReloadNavigation : Boolean;
    FisHistoryNavigation : Boolean;
    Fsignal : TJSAbortSignal;
  public
    function clone: TJSRequest;
    property method : string read Fmethod;
    property url : string read Furl;
    property headers : TJSHeaders read Fheaders;
    property destination : RequestDestination read Fdestination;
    property referrer : string read Freferrer;
    property _referrerPolicy : ReferrerPolicy read FreferrerPolicy;
    property mode : RequestMode read Fmode;
    property credentials : RequestCredentials read Fcredentials;
    property cache : RequestCache read Fcache;
    property redirect : RequestRedirect read Fredirect;
    property integrity : string read Fintegrity;
    property keepalive : Boolean read Fkeepalive;
    property isReloadNavigation : Boolean read FisReloadNavigation;
    property isHistoryNavigation : Boolean read FisHistoryNavigation;
    property signal : TJSAbortSignal read Fsignal;
  end;
  
  { --------------------------------------------------------------------
    TJSResponse
    --------------------------------------------------------------------}

  TJSResponse = class(TJSBody)
  private
    Ftype_ : ResponseType;
    Furl : string;
    Fredirected : Boolean;
    Fstatus : Cardinal;
    Fok : Boolean;
    FstatusText : string;
    Fheaders : TJSHeaders;
    Ftrailer : TJSPromise;
  public
    function error: TJSResponse;
    function redirect(url : string; status : Cardinal): TJSResponse; overload;
    function redirect(url : string): TJSResponse; overload;
    function clone: TJSResponse;
    property type_ : ResponseType read Ftype_;
    property url : string read Furl;
    property redirected : Boolean read Fredirected;
    property status : Cardinal read Fstatus;
    property ok : Boolean read Fok;
    property statusText : string read FstatusText;
    property headers : TJSHeaders read Fheaders;
    property trailer : TJSPromise read Ftrailer;
  end;
  
  { --------------------------------------------------------------------
    TJSWindowOrWorkerGlobalScope
    --------------------------------------------------------------------}
  
  TJSWindowOrWorkerGlobalScope = class
  private
  public
    function fetch(input : RequestInfo; init : TJSRequestInit): TJSPromise; overload;
    function fetch(input : RequestInfo): TJSPromise; overload;
  end;
  
  { --------------------------------------------------------------------
    TJSAbstractWorker
    --------------------------------------------------------------------}
  
  TJSAbstractWorker = class
  private
  public
      onerror : TJSEventHandler;
  end;
  
  { --------------------------------------------------------------------
    TJSWorker
    --------------------------------------------------------------------}
  
  TJSWorker = class(TJSEventTarget)
  private
  public
      onmessage : TJSEventHandler;
      onmessageerror : TJSEventHandler;
      onerror : TJSEventHandler;
    procedure terminate;
    procedure postMessage(message : JSValue; transfer : TTJSObjectDynArray); overload;
    procedure postMessage(message : JSValue; options : TJSPostMessageOptions); overload;
    procedure postMessage(message : JSValue); overload;
  end;
  
  { --------------------------------------------------------------------
    TJSSharedWorker
    --------------------------------------------------------------------}
  
  TJSSharedWorker = class(TJSEventTarget)
  private
    Fport : TJSMessagePort;
  public
      onerror : TJSEventHandler;
    property port : TJSMessagePort read Fport;
  end;
  
  { --------------------------------------------------------------------
    TJSServiceWorker
    --------------------------------------------------------------------}
  
  TJSServiceWorker = class(TJSEventTarget)
  private
    FscriptURL : string;
    Fstate : ServiceWorkerState;
  public
      onstatechange : TJSEventHandler;
      onerror : TJSEventHandler;
    procedure postMessage(message : JSValue; transfer : TTJSObjectDynArray); overload;
    procedure postMessage(message : JSValue; options : TJSPostMessageOptions); overload;
    procedure postMessage(message : JSValue); overload;
    property scriptURL : string read FscriptURL;
    property state : ServiceWorkerState read Fstate;
  end;
  
  { --------------------------------------------------------------------
    TJSServiceWorkerRegistration
    --------------------------------------------------------------------}
  
  TJSServiceWorkerRegistration = class(TJSEventTarget)
  private
    Finstalling : TJSServiceWorker;
    Fwaiting : TJSServiceWorker;
    Factive : TJSServiceWorker;
    FnavigationPreload : TJSNavigationPreloadManager;
    Fscope : string;
    FupdateViaCache : ServiceWorkerUpdateViaCache;
  public
      onupdatefound : TJSEventHandler;
    function update: TJSPromise;
    function unregister: TJSPromise;
    property installing : TJSServiceWorker read Finstalling;
    property waiting : TJSServiceWorker read Fwaiting;
    property active : TJSServiceWorker read Factive;
    property navigationPreload : TJSNavigationPreloadManager read FnavigationPreload;
    property scope : string read Fscope;
    property updateViaCache : ServiceWorkerUpdateViaCache read FupdateViaCache;
  end;
  
  { --------------------------------------------------------------------
    TJSNavigator
    --------------------------------------------------------------------}
  
  TJSNavigator = class
  private
    FserviceWorker : TJSServiceWorkerContainer;
  public
    property serviceWorker : TJSServiceWorkerContainer read FserviceWorker;
  end;
  
  { --------------------------------------------------------------------
    TJSWorkerNavigator
    --------------------------------------------------------------------}
  
  TJSWorkerNavigator = class
  private
    FserviceWorker : TJSServiceWorkerContainer;
  public
    property serviceWorker : TJSServiceWorkerContainer read FserviceWorker;
  end;
  
  { --------------------------------------------------------------------
    TJSServiceWorkerContainer
    --------------------------------------------------------------------}
  
  TJSServiceWorkerContainer = class(TJSEventTarget)
  private
    Fcontroller : TJSServiceWorker;
    Fready : TJSPromise;
  public
      oncontrollerchange : TJSEventHandler;
      onmessage : TJSEventHandler;
      onmessageerror : TJSEventHandler;
    function register(scriptURL : string; options : TJSRegistrationOptions): TJSPromise; overload;
    function register(scriptURL : string): TJSPromise; overload;
    function getRegistration(clientURL : string): TJSPromise; overload;
    function getRegistration: TJSPromise; overload;
    function getRegistrations: TJSPromise;
    procedure startMessages;
    property controller : TJSServiceWorker read Fcontroller;
    property ready : TJSPromise read Fready;
  end;
  
  { --------------------------------------------------------------------
    TJSNavigationPreloadManager
    --------------------------------------------------------------------}
  
  TJSNavigationPreloadManager = class
  private
  public
    function enable: TJSPromise;
    function disable: TJSPromise;
    function setHeaderValue(value : string): TJSPromise;
    function getState: TJSPromise;
  end;
  
  { --------------------------------------------------------------------
    TJSServiceWorkerGlobalScope
    --------------------------------------------------------------------}

  TJSServiceWorkerGlobalScope = class(TJSObject)
  private
    Fclients : TJSClients;
    Fregistration : TJSServiceWorkerRegistration;
  public
      oninstall : TJSEventHandler;
      onactivate : TJSEventHandler;
      onfetch : TJSEventHandler;
      onmessage : TJSEventHandler;
      onmessageerror : TJSEventHandler;
    function skipWaiting: TJSPromise;
    property clients : TJSClients read Fclients;
    property registration : TJSServiceWorkerRegistration read Fregistration;
  end;
  
  { --------------------------------------------------------------------
    TJSClient
    --------------------------------------------------------------------}
  
  TJSClient = class
  private
    Furl : string;
    FframeType : FrameType;
    Fid : string;
    Ftype_ : ClientType;
  public
    procedure postMessage(message : JSValue; transfer : TTJSObjectDynArray); overload;
    procedure postMessage(message : JSValue; options : TJSPostMessageOptions); overload;
    procedure postMessage(message : JSValue); overload;
    property url : string read Furl;
    property _frameType : FrameType read FframeType;
    property id : string read Fid;
    property type_ : ClientType read Ftype_;
  end;
  
  { --------------------------------------------------------------------
    TJSWindowClient
    --------------------------------------------------------------------}
  
  TstringDynArray = array of string;
  TJSWindowClient = class(TJSClient)
  private
    FvisibilityState : string;
    Ffocused : Boolean;
    FancestorOrigins : TstringDynArray;
  public
    function focus: TJSPromise;
    function navigate(url : string): TJSPromise;
    property visibilityState : string read FvisibilityState;
    property focused : Boolean read Ffocused;
    property ancestorOrigins : TstringDynArray read FancestorOrigins;
  end;
  
  { --------------------------------------------------------------------
    TJSClients
    --------------------------------------------------------------------}
  
  TJSClients = class
  private
  public
    function get(id : string): TJSPromise;
    function matchAll(options : TJSClientQueryOptions): TJSPromise; overload;
    function matchAll: TJSPromise; overload;
    function openWindow(url : string): TJSPromise;
    function claim: TJSPromise;
  end;
  
  { --------------------------------------------------------------------
    TJSExtendableEvent
    --------------------------------------------------------------------}
  
  TJSExtendableEvent = class(TJSEvent)
  private
  public
    procedure waitUntil(f : TJSPromise);
  end;
  
  { --------------------------------------------------------------------
    TJSFetchEvent
    --------------------------------------------------------------------}
  
  TJSFetchEvent = class(TJSExtendableEvent)
  private
    Frequest : TJSRequest;
    FpreloadResponse : TJSPromise;
    FclientId : string;
    FresultingClientId : string;
    FreplacesClientId : string;
  public
    procedure respondWith(r : TJSPromise);
    property request : TJSRequest read Frequest;
    property preloadResponse : TJSPromise read FpreloadResponse;
    property clientId : string read FclientId;
    property resultingClientId : string read FresultingClientId;
    property replacesClientId : string read FreplacesClientId;
  end;
  
  { --------------------------------------------------------------------
    TJSExtendableMessageEvent
    --------------------------------------------------------------------}
  
  TJSExtendableMessageEvent = class(TJSExtendableEvent)
  private
    Fdata : JSValue;
    Forigin : string;
    FlastEventId : string;
    Fsource : JSValue;
    Fports : TTJSMessagePortDynArray;
  public
    property data : JSValue read Fdata;
    property origin : string read Forigin;
    property lastEventId : string read FlastEventId;
    property source : JSValue read Fsource;
    property ports : TTJSMessagePortDynArray read Fports;
  end;
  
  
  { --------------------------------------------------------------------
    TJSCache
    --------------------------------------------------------------------}
  
  TRequestInfoDynArray = array of RequestInfo;
  
  TJSCache = class
  private
  public
    function match(request : RequestInfo; options : TJSCacheQueryOptions): TJSPromise; overload;
    function match(request : RequestInfo): TJSPromise; overload;
    function matchAll(request : RequestInfo; options : TJSCacheQueryOptions): TJSPromise; overload;
    function matchAll: TJSPromise; overload;
    function matchAll(request : RequestInfo): TJSPromise; overload;
    function add(request : RequestInfo): TJSPromise;
    function addAll(requests : TRequestInfoDynArray): TJSPromise;
    function put(request : RequestInfo; response : TJSResponse): TJSPromise;
    function delete(request : RequestInfo; options : TJSCacheQueryOptions): TJSPromise; overload;
    function delete(request : RequestInfo): TJSPromise; overload;
    function keys(request : RequestInfo; options : TJSCacheQueryOptions): TJSPromise; overload;
    function keys: TJSPromise; overload;
    function keys(request : RequestInfo): TJSPromise; overload;
  end;
  
  { --------------------------------------------------------------------
    TJSCacheStorage
    --------------------------------------------------------------------}
  
  TJSCacheStorage = class
  private
  public
    function match(request : RequestInfo; options : TJSMultiCacheQueryOptions): TJSPromise; overload;
    function match(request : RequestInfo): TJSPromise; overload;
    function has(cacheName : string): TJSPromise;
    function open(cacheName : string): TJSPromise;
    function delete(cacheName : string): TJSPromise;
    function keys: TJSPromise;
  end;

implementation


{ TJSMessagePort }

procedure TJSMessagePort.close;
begin

end;

procedure TJSMessagePort.postMessage(message: JSValue;
  transfer: TTJSObjectDynArray);
begin

end;

procedure TJSMessagePort.postMessage(message: JSValue;
  options: TJSPostMessageOptions);
begin

end;

procedure TJSMessagePort.postMessage(message: JSValue);
begin

end;

procedure TJSMessagePort.start;
begin

end;

 { TJSHeaders }

procedure TJSHeaders.append(name, value: string);
begin

end;

procedure TJSHeaders.delete(name: string);
begin

end;

function TJSHeaders.get(name: string): string;
begin
  Result := '';
end;

function TJSHeaders.has(name: string): Boolean;
begin
  Result := False;
end;

procedure TJSHeaders.set_(name, value: string);
begin

end;

{ TJSBody }

function TJSBody.arrayBuffer: TJSPromise;
begin
  Result := nil;
end;

function TJSBody.blob: TJSPromise;
begin
  Result := nil;
end;

function TJSBody.formData: TJSPromise;
begin
  Result := nil;
end;

function TJSBody.json: TJSPromise;
begin
  Result := nil;
end;

function TJSBody.text: TJSPromise;
begin
  Result := nil;
end;

{ TJSRequest }

function TJSRequest.clone: TJSRequest;
begin
  Result := nil;
end;

{ TJSResponse }

function TJSResponse.clone: TJSResponse;
begin
  Result := nil;
end;

function TJSResponse.error: TJSResponse;
begin
  Result := nil;
end;

function TJSResponse.redirect(url: string): TJSResponse;
begin
  Result := nil;
end;

function TJSResponse.redirect(url: string; status: Cardinal): TJSResponse;
begin
  Result := nil;
end;

{ TJSWindowOrWorkerGlobalScope }

function TJSWindowOrWorkerGlobalScope.fetch(input: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSWindowOrWorkerGlobalScope.fetch(input: RequestInfo;
  init: TJSRequestInit): TJSPromise;
begin
  Result := nil;
end;

{ TJSWorker }

procedure TJSWorker.postMessage(message: JSValue; transfer: TTJSObjectDynArray);
begin

end;

procedure TJSWorker.postMessage(message: JSValue;
  options: TJSPostMessageOptions);
begin

end;

procedure TJSWorker.postMessage(message: JSValue);
begin

end;

procedure TJSWorker.terminate;
begin

end;

{ TJSServiceWorker }

procedure TJSServiceWorker.postMessage(message: JSValue;
  transfer: TTJSObjectDynArray);
begin

end;

procedure TJSServiceWorker.postMessage(message: JSValue;
  options: TJSPostMessageOptions);
begin

end;

procedure TJSServiceWorker.postMessage(message: JSValue);
begin

end;

{ TJSServiceWorkerRegistration }

function TJSServiceWorkerRegistration.unregister: TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerRegistration.update: TJSPromise;
begin
  Result := nil;
end;

{ TJSServiceWorkerContainer }

function TJSServiceWorkerContainer.getRegistration(
  clientURL: string): TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.getRegistration: TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.getRegistrations: TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.register(scriptURL: string;
  options: TJSRegistrationOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSServiceWorkerContainer.register(scriptURL: string): TJSPromise;
begin
  Result := nil;
end;

procedure TJSServiceWorkerContainer.startMessages;
begin

end;

{ TJSNavigationPreloadManager }

function TJSNavigationPreloadManager.disable: TJSPromise;
begin
  Result := nil;
end;

function TJSNavigationPreloadManager.enable: TJSPromise;
begin
  Result := nil;
end;

function TJSNavigationPreloadManager.getState: TJSPromise;
begin
  Result := nil;
end;

function TJSNavigationPreloadManager.setHeaderValue(value: string): TJSPromise;
begin
  Result := nil;
end;

{ TJSServiceWorkerGlobalScope }

function TJSServiceWorkerGlobalScope.skipWaiting: TJSPromise;
begin
  Result := nil;
end;

{ TJSClient }

procedure TJSClient.postMessage(message: JSValue; transfer: TTJSObjectDynArray);
begin

end;

procedure TJSClient.postMessage(message: JSValue;
  options: TJSPostMessageOptions);
begin

end;

procedure TJSClient.postMessage(message: JSValue);
begin

end;

{ TJSWindowClient }

function TJSWindowClient.focus: TJSPromise;
begin
  Result := nil;
end;

function TJSWindowClient.navigate(url: string): TJSPromise;
begin
  Result := nil;
end;

{ TJSClients }

function TJSClients.claim: TJSPromise;
begin
  Result := nil;
end;

function TJSClients.get(id: string): TJSPromise;
begin
  Result := nil;
end;

function TJSClients.matchAll(options: TJSClientQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSClients.matchAll: TJSPromise;
begin
  Result := nil;
end;

function TJSClients.openWindow(url: string): TJSPromise;
begin
  Result := nil;
end;

{ TJSExtendableEvent }

procedure TJSExtendableEvent.waitUntil(f: TJSPromise);
begin

end;

{ TJSFetchEvent }

procedure TJSFetchEvent.respondWith(r: TJSPromise);
begin

end;

{ TJSCache }

function TJSCache.add(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.addAll(requests: TRequestInfoDynArray): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.delete(request: RequestInfo;
  options: TJSCacheQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.delete(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.keys(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.keys: TJSPromise;
begin
  Result := nil;
end;

function TJSCache.keys(request: RequestInfo;
  options: TJSCacheQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.match(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.match(request: RequestInfo;
  options: TJSCacheQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.matchAll: TJSPromise;
begin
  Result := nil;
end;

function TJSCache.matchAll(request: RequestInfo;
  options: TJSCacheQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.matchAll(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCache.put(request: RequestInfo; response: TJSResponse): TJSPromise;
begin
  Result := nil;
end;

{ TJSCacheStorage }

function TJSCacheStorage.delete(cacheName: string): TJSPromise;
begin
  Result := nil;
end;

function TJSCacheStorage.has(cacheName: string): TJSPromise;
begin
  Result := nil;
end;

function TJSCacheStorage.keys: TJSPromise;
begin
  Result := nil;
end;

function TJSCacheStorage.match(request: RequestInfo;
  options: TJSMultiCacheQueryOptions): TJSPromise;
begin
  Result := nil;
end;

function TJSCacheStorage.match(request: RequestInfo): TJSPromise;
begin
  Result := nil;
end;

function TJSCacheStorage.open(cacheName: string): TJSPromise;
begin
  Result := nil;
end;

end.

